# Copyright 2014 MongoDB, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import logging

logger = logging.getLogger('giza.content.steps.inheritance')

from giza.core.inheritance import InheritableContentError, DataContentBase, DataCache
from giza.content.steps.models import StepData

class StepError(Exception): pass

class StepFile(DataContentBase):
    content_class = StepData

    @property
    def steps(self):
        if not hasattr(self, '_ordered_content'):
            self._ordered_content = []

        if len(self._ordered_content) == 0:
            ret = []

            for step in self.content.values():
                step.resolve(self.data)
                ret.append((step.number, step))

            def sorter(x, y):
                return cmp(x[0], y[0])

            ret.sort(cmp=lambda x,y: cmp(x[0], y[0]))
            self._ordered_content = [ r for idx, r in ret ]

        return self._ordered_content

    def add(self, doc):
        content = super(StepFile, self).add(doc)

        if not hasattr(self, '_step_counter'):
            self._step_counter = 1
        else:
            self._step_counter += 1

        obj = self.content[content.ref]

        if 'number' not in obj:
            obj.number = self._step_counter

class StepDataCache(DataCache):
    content_class = StepFile



# Copyright 2014 MongoDB, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import logging
import shutil
import os
import re

import polib

from giza.translate.utils import TempDir, get_file_list, flip_text_direction
from giza.tools.command import command

'''
This module has functions for translating files. It has a function for
translating any file given a config and it has a functioni for translating
a directory of po files. The po file translator takes the directory,
writes the source language to a file, then translates that to another
file, and then writes that out in the same order as it ingested the
directory. It translates the untranslated entries and empties the
translated ones to isolate the machine translated ones.Thus it is
important not to change the directory structure mid running this program.
MAKE SURE TO COPY THE DIRECTORY OF PO FILES BEFORE TRANSLATING THEM,
anything that was already translated will be deleted by this, this is
intentional. The goal of this module is to make a directory tree with
translations ONLY by Moses, because then those translations can be looked
at separately from approved or human translated sentences.
'''

logger = logging.getLogger('giza.translate.translation')


def translate_file(in_file, out_file,  tconf, protected_file, super_temp=None):
    '''This function translates a given file to another language

    :param string in_file: path to file to be translated
    :param string out_file: path to file where translated output should be written
    :param config tconf: translateconfig object
    :param string protected_file': path to regex file to protect expressions from tokenization
    :param string super_temp: If you have a TempDir context inside of a TempDir
       context, this allows you to not create two. Just pass in the directory of
       the previous temporary directory
    '''

    if out_file is None:
        out_file = in_file + ".translated"

    with TempDir(super_temp=super_temp) as temp:
        logger.info("tempdir: " + temp)
        logger.info("decoding: " + in_file)
        if os.stat(in_file).st_size == 0:
            logger.warning("File is empty")
            open(out_file, "w").close()
            return
        if super_temp is None:
            shutil.copy(in_file, temp)
        in_file = os.path.basename(in_file)

        if protected_file is not None:
            command("{0}/scripts/tokenizer/tokenizer.perl -l en < {4}/{1} > {4}/{1}.tok.en -threads {2} -protected {3}".format(tconf.paths.moses, in_file, tconf.settings.threads, protected_file, temp), logger=logger, capture=True)
        else:
            command("{0}/scripts/tokenizer/tokenizer.perl -l en < {3}/{1} > {3}/{1}.tok.en -threads {2}".format(tconf.paths.moses, in_file, tconf.settings.threads, temp), logger=logger, capture=True)

        command("{0}/scripts/recaser/truecase.perl --model {1}/truecase-model.en < {3}/{2}.tok.en > {3}/{2}.true.en".format(tconf.paths.moses, tconf.paths.aux_corpus_files, in_file, temp), logger=logger, capture=True)
        command("{0}/bin/moses -f {1}/{3}/working/binarised-model/moses.ini < {4}/{2}.true.en > {4}/{2}.true.trans".format(tconf.paths.moses, tconf.paths.project, in_file, tconf.settings.best_run, temp), logger=logger, capture=True)
        command("{0}/scripts/recaser/detruecase.perl < {2}/{1}.true.trans > {2}/{1}.tok.trans".format(tconf.paths.moses, in_file, temp), logger=logger, capture=True)
        command("{0}/scripts/tokenizer/detokenizer.perl -l en < {3}/{1}.tok.trans > {2}".format(tconf.paths.moses, in_file, out_file, temp), logger=logger, capture=True)


def po_file_untranslated_to_text(text_doc, po_file):
    '''This function writes a po file out to a text file

    :param string text_doc: the path to the text document to write the po file sentences to
    :param string po_file: the path to the po_file to write from
    '''
    logger.info("writing out from " + po_file)
    po = polib.pofile(po_file)

    for entry in po.untranslated_entries():
        text_doc.write(entry.msgid.encode('utf-8') + '\n')


def extract_all_untranslated_po_entries(po_file_list, temp_dir):
    '''This function extracts all of the untranslated entries in
    the directory structure

    :param list po_file_list: the list of po files
    :param string temp_dir: the path to the temporary directory

    :returns: the path to the temporary source file
    '''

    with open(temp_dir+"/source", "w") as temp:
        for fn in po_file_list:
            po_file_untranslated_to_text(temp, fn)

    return temp_dir + "/source"


def fill_po_file(target_po_file, translated_lines, start):
    '''This function fills a new po file with the translated lines
    :param string target_po_file: the path to the current po file
    :param list translated_lines: the list of translated lines
    :param int start: the line to start at when writing out this file
    :returns: The start of the sentences for the next file
    '''
    logger.info("writing out to " + target_po_file)
    po = polib.pofile(target_po_file)
    i = -1
    for entry in po:
        if entry.translated() is False:
            i += 1
            entry.msgstr = unicode(translated_lines[start+i].strip(), "utf-8")
        else:
            entry.msgstr = ""

    po.save(target_po_file)
    return start + i + 1


def write_po_files(po_file_list, translated_temp):
    ''' This function writes all of the lines to their po files
    :param list po_file_list: the list of translated lines
    :param string translated_temp: the temproray translated file
    '''
    start = 0
    with open(translated_temp, "r") as trans:
        trans_lines = trans.readlines()

    for fn in po_file_list:
        start = fill_po_file(fn, trans_lines, start)


def translate_po_files(po_path, tconf, protected_file=None):
    ''' This function translates a directory of po files in three steps:
    First it extracts the untranslated entries from every po file into one
    big file. Then it translates that file. Lastly it fills in all of the
    po files in the same order the entries were extracted, removing the text
    from any translated entries.
    :param string po_path: the path to the top level directory of the po_files
    :param config tconf: translation config object
    :param string protected_file: path to file with regexes to protect
    '''

    with TempDir() as temp_dir:
        po_file_list = get_file_list(po_path, ["po", "pot"])
        temp_file = extract_all_untranslated_po_entries(po_file_list, temp_dir)
        trans_file = temp_file + '.translated'
        translate_file(temp_file, trans_file, tconf, protected_file, temp_dir)

        #flips the file if the language is right to left
        if tconf.settings.foreign in ['he', 'ar']:
            flipped_file = trans_file + '.flip'
            flip_text_direction(trans_file, flipped_file)
            trans_file = flipped_file

        write_po_files(po_file_list, trans_file)

def auto_approve_po_entries(po_path):
    ''' This function automatically approves any untranslated sentence in a
    po file that should be identical in the target language. These sentences
    are of the form ``:word:\`link\```
    :param string po_path: the path to the top level directory of the po_files
    '''
    po_file_list = get_file_list(po_path, ["po", "pot"])
    reg = re.compile('^:[a-zA-Z0-9]+:`(?!.*<.*>.*)[^`]*`$')
    for fn in po_file_list:
        po = polib.pofile(fn)
        for entry in po.untranslated_entries():
            match = re.match(reg, entry.msgid.encode('utf-8'))
            if match is not None and match.group() == entry.msgid.encode('utf-8'):
                entry.msgstr = entry.msgid
        po.save()



# Copyright 2014 MongoDB, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
Worker functions that create artifacts for archives of build targets that are
distributed as tarballs (i.e. html sites, manpages, and slides) for offline use.
"""

import os
import logging

logger = logging.getLogger('giza.content.post.archives')

from giza.tools.strings import hyph_concat
from giza.tools.files import copy_if_needed, create_link, tarball

def get_tarball_name(builder, conf):
    if builder == 'link-html':
        fn = conf.project.name + '.tar.gz'
    elif builder == 'link-man':
        fn = "manpages.tar.gz"
    elif builder == 'link-slides':
        fn = hyph_concat(conf.project.name, 'slides') + '.tar.gz'
    elif builder.startswith('man'):
        fn = hyph_concat('manpages', conf.git.branches.current) + '.tar.gz'
    elif builder.startswith('html'):
        fn = hyph_concat(conf.project.name, conf.git.branches.current) + '.tar.gz'
    else:
        fn = hyph_concat(conf.project.name, conf.git.branches.current, builder) + '.tar.gz'

    return os.path.join(conf.paths.projectroot,
                        conf.paths.public_site_output,
                        fn)

def html_tarball(builder, conf):
    copy_if_needed(os.path.join(conf.paths.projectroot,
                                conf.paths.branch_includes, 'hash.rst'),
                   os.path.join(conf.paths.projectroot,
                                conf.paths.branch_output,
                                builder, 'release.txt'))

    tarball_name = get_tarball_name('html', conf)

    tarball(name=tarball_name,
            path=builder,
            cdir=os.path.join(conf.paths.projectroot,
                              conf.paths.branch_output),
            newp=os.path.splitext(os.path.basename(tarball_name))[0])

    link_name = get_tarball_name('link-html', conf)

    if os.path.exists(link_name):
        os.remove(link_name)

    create_link(input_fn=os.path.basename(tarball_name),
                 output_fn=link_name)

def slides_tarball(builder, conf):
    copy_if_needed(os.path.join(conf.paths.projectroot,
                                conf.paths.branch_includes, 'hash.rst'),
                   os.path.join(conf.paths.projectroot,
                                conf.paths.branch_output,
                                builder, 'release.txt'))

    tarball_name = get_tarball_name('slides', conf)

    tarball(name=tarball_name,
            path=builder,
            cdir=os.path.join(conf.paths.projectroot,
                              conf.paths.branch_output),
            newp=os.path.splitext(os.path.basename(tarball_name))[0])

    link_name = get_tarball_name('link-slides', conf)

    if os.path.exists(link_name):
        os.remove(link_name)

    create_link(input_fn=os.path.basename(tarball_name),
                 output_fn=link_name)

def man_tarball(builder, conf):
    tarball_name = get_tarball_name('man', conf)

    tarball(name=tarball_name,
            path=builder,
            cdir=os.path.join(conf.paths.projectroot, conf.paths.branch_output),
            newp=conf.project.name + '-manpages')

    link_name = get_tarball_name('link-man', conf)

    if os.path.exists(link_name):
        os.remove(link_name)

    create_link(input_fn=os.path.basename(tarball_name),
                 output_fn=link_name)



# Copyright 2014 MongoDB, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
Operations for managing and producing artifacts using the ``gettext`` builder,
``sphinx-intl``, and Transifex-based translation workflow.
"""

import logging
import os.path
import sys

import argh

logger = logging.getLogger('giza.operations.tx')

from giza.config.helper import fetch_config
from giza.config.sphinx_config import resolve_builder_path
from giza.core.app import BuildApp
from giza.tools.command import command
from giza.tools.strings import hyph_concat
from giza.tools.files import FileLogger

from sphinx_intl.commands import update_txconfig_resources

#################### Helpers ####################

def tx_resources(conf):
    tx_conf = os.path.join(conf.paths.projectroot,
                           ".tx", 'config')

    with open(tx_conf, 'r') as f:
        resources = [ l.strip()[1:-1]
                      for l in f.readlines()
                      if l.startswith('[')][1:]

    return resources

def logged_command(verb, cmd):
    r = command(cmd, capture=True)
    logger.info('{0}ed {1}'.format(verb, cmd.split(' ')[-1]))

    return r.out

def check_for_orphaned_tx_files(conf):
    tx_conf = os.path.join(conf.paths.projectroot,
                           ".tx", 'config')

    with open(tx_conf, 'r') as f:
        files = [ l.rsplit(' ', 1)[1].strip()
                  for l in f.readlines()
                  if l.startswith('source_file')]

    errs = 0
    for fn in files:
        fqfn = os.path.join(conf.paths.projectroot, fn)

        if not os.path.exists(fqfn):
            errs += 1
            logger.error(fqfn + " does not exist.")

    if errs != 0:
        logger.warning("{0} files configured that don't exist.")
    else:
        logger.info('all configured translation source files exist')

    return errs

#################### Task Generators ####################

def pull_tasks(conf, app):
    resources = tx_resources(conf)

    for page in resources:
        t = app.add('task')
        t.job = logged_command
        t.args = ('pull', ' '.join([ 'tx', 'pull', '-l', lang, '-r', page]))
        t.description = 'pulling {0} from transifex client'.format(page)

def push_tasks(conf, app):
    resources = tx_resources(conf)

    for page in resources:
        t = app.add('task')
        t.job = logged_command
        t.args = ('pull', ' '.join([ 'tx', 'pull', '-l', lang, '-r', page]))
        t.description = 'pulling {0} from transifex client'.format(page)

def update(conf):
    logger.info('updating translation artifacts. Long running.')

    project_name = conf.project.title.lower().split()
    if conf.project.edition is not None and conf.project.edition != conf.project.name:
        project_name.append(conf.project.edition)

    project_name = hyph_concat(*project_name)

    logger.info('starting translation upload with sphinx-intl')

    flogger = FileLogger(logger)
    update_txconfig_resources(transifex_project_name=project_name,
                              locale_dir=conf.paths.locale,
                              pot_dir=os.path.join(conf.paths.locale, 'pot'),
                              out=flogger)

    logger.info('sphinx-intl: updated pot directory')

#################### Commands ####################

@argh.named('check')
@argh.expects_obj
def check_orphaned(args):
    conf = fetch_config(args)

    check_for_orphaned_tx_files(conf)

@argh.arg('--edition', '-e')
@argh.arg('--language', '-l')
@argh.named('update')
@argh.expects_obj
def update_translations(args):
    conf = fetch_config(args)

    update(conf)
    check_for_orphaned_tx_files(conf)

@argh.named('pull')
@argh.expects_obj
def pull_translations(args):
    conf = fetch_config(args)
    app = BuildApp(conf)

    pull_tasks(conf, app)
    app.run()

@argh.arg('--edition', '-e')
@argh.arg('--language', '-l')
@argh.named('push')
@argh.expects_obj
def push_translations(args):
    conf = fetch_config(args)
    app = BuildApp(conf)

    push_tasks(conf, app)

    update(conf)
    app.run()



# 2014 MongoDB, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import logging

logger = logging.getLogger('giza.config.redirects')

from giza.config.base import ConfigurationBase
from giza.content.helper import edition_check

def redirect_path_spec_normalization(value):
        o = []
        if not value.startswith('/'):
            o.append('/')

        if value.endswith('/'):
            value = value[:-1]

        o.append(value)

        return ''.join(o)

class RedirectSpecification(ConfigurationBase):
    _option_registry = ['type', 'external', 'edition']

    def __gettattr__(self, key):
        if key == 'from':
            key = 'from_loc'

        super(RedirectSpecification, self).__getattr__(key)

    def __setattr__(self, key, value):
        if key == 'from':
            key = 'from_loc'

        super(RedirectSpecification, self).__setattr__(key, value)

    @property
    def from_loc(self):
        return ''.join([self.output[0], self.state['from']])

    @from_loc.setter
    def from_loc(self, value):
        self.state['from'] = redirect_path_spec_normalization(value)

    @property
    def to(self):
        return ''.join([self.output[1], self.state['to']])

    @to.setter
    def to(self, value):
        self.state['to'] = redirect_path_spec_normalization(value)

    @property
    def code(self):
        return self.state['code']

    @code.setter
    def code(self, value):
        if value in (301, 302, 303, 307, 308):
            self.state['code'] = value
        else:
            raise TypeError

    @property
    def output(self):
        left, right = self.state['output']

        if left in ('', '/'):
            pass
        elif left != '/' and not left.startswith('/') and not left.startswith('http'):
            left = '/' + left

        if right == '/':
            right = ''
        elif not right.startswith('/') and not right.startswith('http'):
            right = '/' + right

        return left, right

    @output.setter
    def output(self, value):
        if isinstance(value, dict):
            o = value.items()[0]
        elif isinstance(value, tuple) and len(value) == 2:
            o = value
        elif not isinstance(value, list):
            o = (value, value)
        else:
            raise TypeError

        self.state['output'] = o

    def dict(self):
        return {
            'from': self.state['from'],
            'to': self.state['to'],
            'code': self.code,
            'output': self.output,
            'external': self.external if 'external' in self.state else ''
        }

class HtaccessData(list):
    def append(self, item):
        self.insert(-1, item)

    def extend(self, items):
        for item in items:
            self.insert(-1, item)

    def insert(self, index, item):
        if item is None:
            return
        elif edition_check(item, self.conf) is False:
            return

        outputs = resolve_outputs_for_redirect(item['outputs'], self.conf)

        for doc in process_redirect_inputs(outputs, item):
            super(HtaccessData, self).insert(index, RedirectSpecification(doc))

def is_computed_output(key):
    if isinstance(key, (tuple, list)):
        key = key[0]

    if key == 'all' or key.startswith('before') or key.startswith('after'):
        return True
    else:
        return False

########## Redirect Resolution ##########

# the following three private functions are refactored components of
# ``resolve_output_for_redirect()``

def _add_outputs_to_computed(computed, keyword, base, conf):
    if keyword == 'all':
        computed.extend(conf.git.branches.published)
    elif keyword == 'before':
        computed.extend(conf.git.branches.published[conf.git.branches.published.index(base):])
    elif keyword == 'after':
        computed.extend(conf.git.branches.published[:conf.git.branches.published.index(base)])

def _render_key(sub_key, left_base, right_base):
    if sub_key == left_base:
        left = sub_key
    else:
        left = '/'.join([left_base, sub_key])

    if sub_key == right_base:
        right = sub_key
    else:
        right = '/'.join([right_base, sub_key])

    return left, right

def _get_redirect_base_paths(computed, out, conf):
    if out == 'all':
        out_key = out_value = ''
        _add_outputs_to_computed(computed, 'all', 0, conf)
    elif isinstance(out, dict):
        out_key, out_value = out.items()[0]
        if isinstance(out_value, dict):
            # for mms where from/to paths are mapped differently
            if '-' in out_key:
                keyword, base = out_key.split('-', 1)
            else:
                keyword = out_key
                base = ''
            out_key, out_value = out_value.items()[0]
            _add_outputs_to_computed(computed, keyword, base, conf)
    else:
        if out == []:
            out_key = out_value = ''
        elif is_computed_output(out):
            keyword, base = out.split('-', 1)
            out_key = out_value = ''
            _add_outputs_to_computed(computed, keyword, base, conf)
        elif isinstance(out, tuple):
            out_key, out_value = out
        else:
            out_key = out_value = out

    return out_key, out_value

# The following functions describe the process for inserting documents into the
# HtaccessData list and are called in HtaccessData.list()

def resolve_outputs_for_redirect(outputs, conf):
    if 'integration' in conf.system.files.data:
        shadows = conf.system.files.data.integration['base']['links']
    else:
        shadows = []

    expanded_outputs = []
    for out in outputs:
        computed = []

        out_key, out_value = _get_redirect_base_paths(computed, out, conf)

        for shadow in shadows:
            key, value = shadow.items()[0]
            if value == out_value:
                expanded_outputs.extend((value, key))

        expanded_outputs.extend([ _render_key(o, out_key, out_value) for o in computed ])

    outputs.extend(expanded_outputs)
    return outputs

def process_redirect_inputs(outputs, item):
    docs = []

    if len(outputs) == 0:
        docs.append(item)
    elif len(outputs) == 1:
        item['output'] = item['outputs'][0]
        del item['outputs']
        docs.append(item)
    else:
        for out in item['outputs']:
            if isinstance(out, dict):
                output = out.items()[0]
            elif isinstance(out, tuple) and len(out) == 2:
                output = out
            else:
                output = (out, out)

            if is_computed_output(output[0]):
                continue

            redir = { 'output': output }
            redir.update(item)
            del redir['outputs']
            docs.append(redir)

    return docs



# Copyright 2014 MongoDB, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
Helpers and wrappers for performing common maintenance operations on a
repository with git.
"""

import logging
import os
from tempfile import NamedTemporaryFile

import argh
from sphinx.application import Sphinx, ENV_PICKLE_FILENAME
from sphinx.builders.html import get_stable_hash

from giza.core.app import BuildApp
from giza.core.git import GitRepo
from giza.config.helper import fetch_config
from giza.tools.command import command
from giza.config.sphinx_config import avalible_sphinx_builders

logger = logging.getLogger('giza.operations.git')

@argh.arg('--patch', '-p', nargs='*', dest='git_objects')
@argh.arg('--branch', '-b', default=None, dest='git_branch')
@argh.arg('--signoff', '-s', default=False, action='store_true', dest='git_sign_patch')
@argh.named('am')
@argh.expects_obj
def apply_patch(args):
    c = fetch_config(args)

    g = GitRepo(c.paths.projectroot)

    if c.runstate.git_branch is None:
        c.runstate.git_branch = g.current_branch()

    with g.branch(c.runstate.git_branch):
        g.am(patches=c.runstate.git_objects,
             repo='/'.join(['https://github.com', c.git.remote.upstream]),
             sign=c.runstate.git_sign_patch)

@argh.arg('--branch', '-b', default=None, dest='git_branch')
@argh.named('update')
@argh.expects_obj
def pull_rebase(args):
    c = fetch_config(args)

    g = GitRepo(c.paths.projectroot)

    if c.runstate.git_branch is None:
        c.runstate.git_branch = g.current_branch()

    with g.branch(c.runstate.git_branch):
        g.update()

@argh.arg('--branch', '-b', default=None, dest='git_branch')
@argh.arg('--commits', '-c', nargs='*', dest='git_objects')
@argh.named('cp')
@argh.expects_obj
def cherry_pick(args):
    c = fetch_config(args)

    g = GitRepo(c.paths.projectroot)

    if c.runstate.git_branch is None:
        c.runstate.git_branch = g.current_branch()

    with g.branch(c.runstate.git_branch):
        g.cherry_pick(c.runstate.git_objects)

@argh.arg('--branch', '-b', default=None, dest='git_branch')
@argh.expects_obj
def merge(args):
    c = fetch_config(args)

    g = GitRepo(c.paths.projectroot)

    from_branch = g.current_branch()
    branch_name = str(id(c.runstate.git_branch))

    g.checkout_branch(branch_name, c.runstate.git_branch)

    try:
        g.checkout(branch_name)
        g.rebase(from_branch)
        g.checkout(from_branch)
        g.merge(c.runstate.git_branch)
        logger.info('rebased and merged {0} into {1}'.format(c.runstate.git_branch, from_branch))
    except Exception as e:
        logger.warning('error attempting to merge branch: ' + c.runstate.git_branch)
        logger.error(e)
    finally:
        if g.current_branch != from_branch:
            g.checkout(from_branch)

        g.remove_branch(branch_name, force=False)

@argh.expects_obj
@argh.named("setup-branches")
def setup_branches(args):
    conf = fetch_config(args)

    g = GitRepo(c.paths.projectroot)

    if 'upstream' in g.remotes():
        remote = 'upstream'
    else:
        remote = 'origin'

    for pbranch in conf.git.branches.published:
        if g.branch_exists(pbranch):
            continue
        else:
            tracking_branch = '/'.join([remote, pbranch])
            g.create_branch(pbranch, tracking=tracking_branch)
            logger.info('created branch "{0}" tracking "{1}"'.format(pbranch, tracking_branch))


@argh.expects_obj
@argh.named("create-branch")
@argh.arg('git_branch')
def create_branch(args):
    """
    Takes a single branch name and (if necessary) creates a new branch. Then,
    populates the ``build/<branch>`` directory for the new branch using either
    the parent branch or ``master``. Safe to run multiple times (after a rebase)
    to update the build cache from master.

    Also calls ``fix_build_environment()`` to tweak the new build output to
    update hashes and on-disk copies of the environment to prevent unnecessary
    full-rebuilds from sphinx.
    """

    conf = fetch_config(args)
    app = BuildApp(conf)
    app.pool = 'process'

    g = GitRepo(conf.paths.projectroot)

    branch = conf.runstate.git_branch
    base_branch = g.current_branch()

    if base_branch == branch:
        base_branch = 'master'
        logger.warning('seeding build data for branch "{0}" from "master"'.format(branch))

    branch_builddir = os.path.join(conf.paths.projectroot,
                                   conf.paths.output, branch)

    base_builddir = os.path.join(conf.paths.projectroot,
                                   conf.paths.output, base_branch)

    if g.branch_exists(branch):
        logger.info('checking out branch "{0}"'.format(branch))
    else:
        logger.info('creating and checking out a branch named "{0}"'.format(branch))

    g.checkout_branch(branch)

    cmd = "rsync -r --times --checksum {0}/ {1}".format(base_builddir, branch_builddir)
    logger.info('seeding build directory for "{0}" from "{1}"'.format(branch, base_branch))
    command(cmd)
    logger.info('branch creation complete.')

    # get a new config here for the new branch
    conf = fetch_config(args)
    for builder in [ b
                     for b in avalible_sphinx_builders()
                     if os.path.isdir(os.path.join(conf.paths.projectroot, conf.paths.branch_output, b)) ]:
        t = app.add('task')
        t.job = fix_build_environment
        t.args = (builder, conf)
        t.target = True
        t.description = "fix up sphinx environment for builder '{0}'".format(builder)

    app.run()

def fix_build_environment(builder, conf):
    """
    Given a builder name and the conf object, this function fixes the build
    artifacts for the current build to prevent a full rebuild. Currently
    re-pickles the environment and dumps the ``.buildinfo`` file in the build
    directory with the correct hashes.
    """

    fn = os.path.join(conf.paths.projectroot, conf.paths.branch_output, builder, '.buildinfo')
    logger.info('updating cache for: ' + builder)

    if not os.path.isfile(fn):
        return

    doctree_dir = os.path.join(conf.paths.projectroot, conf.paths.branch_output, "doctrees-" + builder)

    sphinx_app = Sphinx(
        srcdir=os.path.join(conf.paths.projectroot, conf.paths.branch_output, "source"),
        confdir=conf.paths.projectroot,
        outdir=os.path.join(conf.paths.projectroot, conf.paths.branch_output, builder),
        doctreedir=doctree_dir,
        buildername=builder,
        status=NamedTemporaryFile(),
        warning=NamedTemporaryFile())

    sphinx_app.env.topickle(os.path.join(doctree_dir, ENV_PICKLE_FILENAME))

    with open(fn, 'r') as f:
        lns = f.readlines()
        tags_hash_ln = None
        for ln in lns:
            if ln.startswith('tags'):
                tags_hash_ln = ln
                break

        if tags_hash_ln == None:
            tags_hash_ln = 'tags: '  + get_stable_hash(sorted(sphinx_app.tags))

    with open(fn, 'w') as f:
        f.write('# Sphinx build info version 1')
        f.write('\n\n') ## current format requires an extra line here.
        f.write('config: ' + get_stable_hash(dict((name, sphinx_app.config[name])
                                                  for (name, desc) in sphinx_app.config.values.items()
                                                  if desc[1] == 'html')))
        f.write('\n')
        f.write(tags_hash_ln)
        f.write('\n')



import os
import yaml
import requests
import logging

from requests.auth import HTTPDigestAuth

logger = logging.getLogger('giza.corp')

from giza.config.credentials import CredentialsConfig

def corp_api_call(endpoint, conf):
    credentials = CredentialsConfig(conf.site.credentials).corp

    url = conf.site.corp + "/api/" + endpoint
    headers = {'accept': 'application/json'}
    response = requests.get(url,
                            auth=HTTPDigestAuth(credentials.username, credentials.password),
                            headers=headers)

    return response.json()

def get_contributor_list(conf):
    contributors = corp_api_call('contributors', conf)['contributors']

    c_github = list({ str(c['github_username']) for c in contributors
                      if ('github_username' in c and
                          c['github_username'] is not None and
                          c['github_username'] != u'') })

    return c_github



# Copyright 2014 MongoDB, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
:mod:`app` holds the :class:`~giza.app.BuildApp()` class that provides an
organizing framework for running larger sequences of operations.
"""

import logging
import random

logger = logging.getLogger('giza.app')

from giza.core.pool import ThreadPool, ProcessPool, SerialPool, WorkerPool, EventPool
from giza.config.main import Configuration
from giza.config.helper import new_skeleton_config

from giza.core.task import Task, MapTask

class BuildApp(object):
    """
    A unit of work. :class:`~giza.app.BuildApp()` instances possess queues of
    :class:`~giza.task.Task()` objects and sub-\ :class:`~giza.app.BuildApp()`
    objects that describe a build process. Groups of :class:`~giza.task.Task()`
    objects may execute in parallel, while any :class:`~giza.app.BuildApp()`
    objects in the queue execute in isolation after proceeding group of
    :class:`~giza.task.Task()` operations complete. Mix
    :class:`~giza.app.BuildApp()` and :class:`~giza.task.Task()` operations to
    control task ordering.

    The results of all operations are accessible in the
    :attr:`~giza.app.BuildApp.results`, which largely preserves the ordering of the
    insertion of operations into the queue. Unlike the queue,
    :attr:`~giza.app.BuildApp.results` contains the result of each operation in
    an embedded :class:`~giza.app.BuildApp()` in the order that each task was
    added to the embedded :class:`~giza.app.BuildApp()` instance.

    :class:`~giza.app.BuildApp()` are reusable: after running all operations in
    the queue, the queue resets. However, results do not reset.
    """

    def __init__(self, conf=None):
        """
        :param Configuration conf: A top level
           :class:`~giza.config.main.Configuration` object.
        """

        self.conf = new_skeleton_config(conf)
        self.queue = []
        self.results = []
        self.worker_pool = None
        self.default_pool = self.conf.runstate.runner
        self.randomize = False

        self.pool_mapping = {
            'thread': ThreadPool,
            'process': ProcessPool,
            'event': EventPool,
            'serial': SerialPool
        }
        self.pool_types = tuple([ self.pool_mapping[p] for p in self.pool_mapping ])

        self.needs_rebuild = True
        self.root_app = True

        self.target = None
        self.dependency = None

    @property
    def dependency(self):
        return self._dependency

    @dependency.setter
    def dependency(self, value):
        self._dependency = value

    @property
    def target(self):
        return self._target

    @target.setter
    def target(self, value):
        self._target = value

    def define_dependency_node(self, target, dependency):
        self.target = target
        self.dependency = dependency

    def reset(self):
        self.queue = []
        self.results = []

    @property
    def pool(self):
        if self.worker_pool is None:
            self.create_pool()

        return self.worker_pool

    @pool.setter
    def pool(self, value=None):
        self.create_pool(value)

    def create_pool(self, pool=None):
        if pool is None:
            if self.worker_pool is None:
                pool = self.default_pool
            else:
                pool = self.worker_pool

        if self.worker_pool is not None:
            logger.debug('not creating a pool because one already exists. ({0}, {1})'.format(pool, type(pool)))
            return

        if isinstance(self.worker_pool, self.pool_mapping[self.default_pool]):
            return
        elif self.worker_pool is None:
            if self.is_pool(pool) and self.worker_pool is None:
                self.worker_pool = pool
            elif pool in self.pool_types:
                self.worker_pool = pool(self.conf)
            elif self.is_pool_type(pool) and self.worker_pool is None:
                self.worker_pool = self.pool_mapping[pool](self.conf)
            else:
                self.worker_pool = self.pool_mapping[self.default_pool](self.conf)
        else:
            raise TypeError("pool {0} of type {1} is invalid".format(pool, type(pool)))

    def is_pool(self, pool):
        if isinstance(pool, self.pool_types):
            return True
        else:
            return False

    def is_pool_type(self, value):
        if value in self.pool_mapping:
            return True
        else:
           return False

    @property
    def queue_has_apps(self):
        num_apps = len([ True for t in self.queue if isinstance(t, BuildApp)])

        if num_apps >= 1:
            return True
        else:
            return False

    def close_pool(self):
        if self.is_pool(self.worker_pool) and not isinstance(self.worker_pool, SerialPool):
            self.worker_pool.close()
            self.worker_pool = None

    def add(self, task=None):
        """
        Adds a new :class:`~giza.app.BuildApp()` or :class:`~giza.task.Task()`
        to the :class:`~giza.app.BuildApp()` object.

        :param string,Task,BuildApp task: Optional. If not specified,
           :meth:`~giza.app.BuildApp.add()` creates and returns a new
           :class:`~giza.task.Task()` object. You can pass the string ``task``
           or the class :class:`~giza.task.Task` to explicitly create a new
           Task, or pass an existing :class:`~giza.task.Task()` instance to add
           that task to the :class:`~giza.app.BuildApp()` instance. You can also
           pass the string ``app`` or the :class:`~giza.app.BuildApp` class, to
           create and add new :class:`~giza.app.BuildApp()`: pass an existing
           :class:`~giza.app.BuildApp()`  instance to add that that operation
           grouping to the queue.

        :returns: A reference to a :class:`~giza.app.BuildApp()` or
           :class:`~giza.task.Task()` object in the :class:`~giza.app.BuildApp()`

        :raises: :exc:`TypeError` if the ``task`` argument is invalid.

        """

        if task is None or task in (Task, 'task'):
            t = Task()
            t.conf = self.conf
            self.queue.append(t)
            return t
        elif task in (MapTask, 'map'):
            t = MapTask()
            t.conf = self.conf
            self.queue.append(t)
            return t
        elif task in (BuildApp, 'app'):
            self.create_pool()
            t = BuildApp(self.conf)
            t.pool = self.pool
            t.root_app = False
            self.queue.append(t)
            return t
        else:
            if isinstance(task, Task):
                if task.conf is None:
                    task.conf = self.conf

                self.queue.append(task)
                return task
            elif isinstance(task, BuildApp):
                self.create_pool()
                task.root_app = False
                task.pool = self.pool
                self.queue.append(task)
                return task
            else:
                raise TypeError('invalid task type')

    def _run_single(self, j):
        if isinstance(j, BuildApp):
            if j.pool is None:
                j.pool = self.pool

            self.results.extend(j.run())
        elif isinstance(j, MapTask):
            self.results.extend(self.pool.runner([j]))
        elif isinstance(j, Task):
            self.results.append(j.run())
        else:
            raise TypeError

    def _run_mixed_queue(self):
        group = [ ]

        for task in self.queue:
            if not isinstance(task, BuildApp):
                group.append(task)
            else:
                if len(group) == 1:
                    j = group[0]
                    if isinstance(j, MapTask):
                        self.results.extend(self.pool.runner([j]))
                    else:
                        self.results.append(j.run())
                        group = []
                elif len(group) > 1:
                    if self.randomize is True:
                        random.shuffle(group)

                    self.results.extend(self.pool.runner(group))
                    group = []

                if task.pool is None:
                    task.pool = self.pool

                if isinstance(task, MapTask):
                    self.results.extend(self.pool.runner([task]))
                elif isinstance(task, Task):
                    self.results.append(task.run())
                else:
                    self.results.extend(task.run())

        if len(group) != 0:
            self.results.extend(self.pool.runner(group))

    def run(self):
        "Executes all tasks in the :attr:`~giza.app.BuildApp.queue`."

        if len(self.queue) == 1:
            self._run_single(self.queue[0])
        elif self.queue_has_apps is True:
            self._run_mixed_queue()
        else:
            if self.randomize is True:
                random.shuffle(self.queue)
            self.results.extend(self.pool.runner(self.queue))

        self.queue = []
        return self.results



# Copyright 2014 MongoDB, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
Contains schema, type checking, and validation structures for the data in
example files.
"""

import sys
import logging

logger = logging.getLogger('giza.content.examples.models')

from giza.config.base import ConfigurationBase
from giza.core.inheritance import InheritableContentBase
from giza.content.helper import get_all_languages

if sys.version_info >= (3, 0):
    basestring = str

class ExampleData(InheritableContentBase):
    @property
    def collection(self):
        if 'collection' not in self.state:
            return None
        else:
            return self.state['collection']

    @collection.setter
    def collection(self, value):
        self.state['collection'] = value

    @property
    def name(self):
        return self.collection

    @property
    def ref(self):
        return self.collection

    @property
    def documents(self):
        return self.state['documents']

    @documents.setter
    def documents(self, value):
        if isinstance(value, list):
            self.state['documents'] = value
        else:
            raise TypeError('{0} is not a list'.format(value))

    @property
    def options(self):
        if 'options' not in self.state:
            return ExampleOptions()
        else:
            return self.state['options']

    @options.setter
    def options(self, value):
        self.state['options'] = ExampleOptions(value)

class ExampleOptions(ConfigurationBase):
    @property
    def show_title(self):
        if 'title' not in self.state:
            return True
        else:
            return self.state['title']

    @show_title.setter
    def show_title(self, value):
        if isinstance(value, bool):
            self.state['title'] = value
        else:
            raise TypeError

    @property
    def show_collection(self):
        if 'collection' not in self.state:
            return True
        else:
            return self.state['collection']

    @show_collection.setter
    def show_collection(self, value):
        if isinstance(value, bool):
            self.state['collection'] = value
        else:
            raise TypeError

class ExampleOperationBlock(ConfigurationBase):
    _option_registry = ['pre', 'post', 'final']

    @property
    def code(self):
        return self.state['code']

    @code.setter
    def code(self, value):
        if isinstance(value, list):
            self.state['code'] = value
        else:
            self.state['code'] = value.split('\n')

    @property
    def language(self):
        return self.state['language']

    @language.setter
    def language(self, value):
        if value in get_all_languages():
            self.state['language'] = value
        else:
            m = '{0} is not a supported language'.format(value)
            logger.error(m)
            TypeError(m)

class ExampleCase(InheritableContentBase):
    @property
    def operation(self):
        return self.state['operation']

    @operation.setter
    def operation(self, value):
        if isinstance(value, list):
            self.state['operation'] = [ ExampleOperationBlock(doc)
                                        for doc in value ]
        elif isinstance(value, dict):
            self.state['operation'] = [ ExampleOperationBlock(value) ]
        else:
            m = 'unable to create operation block from {0}'.format(value)
            logger.error(m)
            TypeError(m)

    name = operation
    @property
    def results(self):
        return self.state['results']

    @results.setter
    def results(self, value):
        if value is None:
            self.state['results'] = value
        elif isinstance(value, list):
            v = []
            for ln in value:
                v.extend(ln.split('\n'))

            self.state['results'] = v
        else:
            self.state['results'] = value.split('\n')



# Copyright 2014 MongoDB, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os
import logging

logger = logging.getLogger('giza.content.steps.tasks')

from giza.tools.files import expand_tree, verbose_remove
from giza.content.steps.inheritance import StepDataCache
from giza.content.steps.views import render_steps

def write_steps(steps, fn, conf):
    content = render_steps(steps, conf)
    content.write(fn)
    logger.info('wrote steps to: '  + fn)

def get_step_fn_prefix(conf):
    return os.path.join(conf.paths.projectroot, conf.paths.branch_includes, 'steps')

def step_outputs(conf):
    include_dir = os.path.join(conf.paths.projectroot, conf.paths.branch_includes)
    fn_prefix = get_step_fn_prefix(conf)

    return [ fn for fn in
             expand_tree(include_dir, 'yaml')
             if fn.startswith(fn_prefix) ]

def step_tasks(conf, app):
    fn_prefix = get_step_fn_prefix(conf)
    step_sources = step_outputs(conf)
    s = StepDataCache(step_sources, conf)

    if len(step_sources) and not os.path.isdir(fn_prefix):
        os.makedirs(fn_prefix)

    for fn, stepf in s.file_iter():
        basename = fn[len(fn_prefix)+1:-5]

        out_fn = os.path.join(conf.paths.projectroot,
                              conf.paths.branch_includes,
                              'steps', basename) + '.rst'

        t = app.add('task')
        t.target = out_fn
        t.dependency = fn
        t.job = write_steps
        t.args = (stepf, out_fn, conf)
        t.description = 'generate an stepfile for ' + fn

def step_clean(conf, app):
    for fn in step_outputs(conf):
        task = app.add('task')
        task.job = verbose_remove
        task.args = [fn]
        task.description = 'removing {0}'.format(fn)



# Copyright 2014 MongoDB, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import logging
import sys
import os

from giza.config.base import ConfigurationBase, RecursiveConfigurationBase
logger = logging.getLogger('giza.config.translate')

class TranslateConfig(RecursiveConfigurationBase):
    def __init__(self, input_obj, conf):
        if isinstance(input_obj, list):
            logger.error("Config doesn't exist")
            raise TypeError
        else:
            super(TranslateConfig,self).__init__(input_obj, conf)

    @property
    def settings(self):
        return self.state['settings']

    @settings.setter
    def settings(self, value):
        self.state['settings'] = SettingsConfig(value)

    @property
    def train(self):
        return self.state['train']

    @train.setter
    def train(self, value):
        self.state['train'] = CorpusTypeConfig(value)

    @property
    def tune(self):
        return self.state['tune']

    @tune.setter
    def tune(self, value):
        self.state['tune'] = CorpusTypeConfig(value)

    @property
    def test(self):
        return self.state['test']

    @test.setter
    def test(self, value):
        self.state['test'] = CorpusTypeConfig(value)

    @property
    def training_parameters(self):
        return self.state['training_parameters']

    @training_parameters.setter
    def training_parameters(self, value):
        self.state['training_parameters'] = TrainingParametersConfig(value)

    @property
    def paths(self):
        return self.state['paths']

    @paths.setter
    def paths(self, value):
        self.state['paths'] = PathsConfig(value)


class SettingsConfig(ConfigurationBase):
    _option_registry = ['foreign', 'threads', 'pool_size', 'email',
                        'phrase_table_name', 'reordering_name', 'best_run']

class PathsConfig(ConfigurationBase):

    @property
    def moses(self):
        return self.state['moses']

    @moses.setter
    def moses(self, value):
        self.state['moses'] = os.path.expanduser(value)
        if os.path.exists(self.state['moses']) is False:
            raise TypeError(value + ' does not exist')

    @property
    def irstlm(self):
        return self.state['irstlm']

    @irstlm.setter
    def irstlm(self, value):
        self.state['irstlm'] = os.path.expanduser(value)
        if os.path.exists(self.state['irstlm']) is False:
            raise TypeError(value + ' does not exist')

    @property
    def aux_corpus_files(self):
        return self.state['aux_corpus_files']

    @aux_corpus_files.setter
    def aux_corpus_files(self, value):
        self.state['aux_corpus_files'] = os.path.expanduser(value)
        if os.path.exists(self.state['aux_corpus_files']) is False:
            raise TypeError(value + ' does not exist')

    @property
    def project(self):
        return self.state['project']

    @project.setter
    def project(self, value):
        self.state['project'] = os.path.expanduser(value)
        if os.path.exists(self.state['project']) is False:
            raise TypeError(value + ' project')

class CorpusTypeConfig(ConfigurationBase):
    _option_registry = ['name']
    @property
    def dir(self):
        return self.state['dir']

    @dir.setter
    def dir(self, value):
        self.state['dir'] = os.path.expanduser(value)
        if os.path.exists(self.state['dir']) is False:
            raise TypeError(value + ' does not exist')

class TrainingParametersConfig(ConfigurationBase):
    _option_registry = ['alignment', 'max_phrase_length', 'order',
                        'reordering_directionality', 'reordering_language', 'reordering_modeltype',
                        'reordering_orientation', 'score_options', 'smoothing']

    def __init__(self, input_obj):
        input_obj = self.transform(input_obj)
        super(TrainingParametersConfig, self).__init__(input_obj)

    def transform(self, input_obj):
        for k, param in input_obj.items():
            if isinstance(param, list) is False:
                input_obj[k] = [param]
        return input_obj



# Copyright 2014 MongoDB, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
Content migration and transformation engine that makes it possible to import and
reuse content from repository, or project in another. Migration specifications
can also include transformation specifications to trim content from a source
file, append content to a file, or apply a regular expression substitution.

Developed for the Primer/Getting Started section. Configuration files are all
``.yaml`` files in the *config* directory that have the substring "migration" in
them. Uses the functions in :mod:`giza.tools.transformation` and
:mod:`giza.tools.files` to perform the action, and just adds migration and
transformation tasks to a :class:`giza.core.app.BuildApp()` instance.
"""

import os
import logging

logger = logging.getLogger('giza.content.primer')

from giza.tools.files import copy_if_needed, copy_always, expand_tree, verbose_remove
from giza.tools.serialization import ingest_yaml_list
from giza.tools.transformation import post_process_tasks, truncate_file, append_to_file

def get_migration_specifications(conf):
    return [ fn for fn in expand_tree(os.path.join(conf.paths.projectroot,
                                                   conf.paths.builddata))
             if  conf.project.name in os.path.basename(fn) and 'migrations' in fn ]

def directory_expansion(source_path, page, conf):
    new_page = { 'sources': expand_tree(source_path, None)}
    del page['source']

    if 'source_dir' in page:
        new_page['source_dir'] = page['source_dir']

    pages = []

    for p in convert_multi_source(new_page):
        p.update(page)
        p['target'] = os.path.join(conf.paths.projectroot,
                                   conf.paths.source,
                                   p['target'],
                                   p['source'][len(source_path)+1:])

        pages.append(p)


    return pages

def convert_multi_source(page):
    pages = [ { 'source': source }
              for source in page['sources'] ]

    for k in ('source', 'sources'):
        if k in page:
            del page[k]

    r = []
    for p in pages:
        p.update(page)
        r.append(p)

    return r


########## Path Normalization Helper Functions

def fix_migration_paths(page):
    if 'target' not in page:
        page['target'] = page['source']

    if 'override' not in page:
        page['override'] = True

    if page['target'].endswith('.txt') and page['override'] is True:
        msg = '({0}) imported files cannot end with ".txt", changing to ".rst"'
        logger.warning(msg.format(page['source']))
        page['target'] = page['target'].replace('.txt', '.rst')

    return page

def trim_leading_slash_from_pages(page):
    for field  in ['source', 'target']:
        if page[field].startswith('/'):
            page[field] = page[field][1:]

    return page

def resolve_page_path(page, conf):
    if page['target'].startswith('/'):
        fq_target = page['target']
    elif '{root}' in page['target']:
        fq_target = page['target'].format(root=conf.paths.projectroot)
    else:
        fq_target = os.path.join(conf.paths.projectroot, conf.paths.source, page['target'])

    if page['source'].startswith('/'):
        fq_source = page['source']
    elif 'source_dir' in page:
        fq_source = os.path.abspath(os.path.join(conf.paths.projectroot, page['source_dir'], page['source']))
    else:
        fq_source = os.path.abspath(os.path.join(conf.paths.projectroot, '..', 'source', page['source']))

    return fq_target, fq_source

########## Main Migration Operations and Task Generator

def primer_migration_tasks(conf, app):
    "Migrates all manual files to primer according to the spec. As needed."

    migration_paths = get_migration_specifications(conf)

    if len(migration_paths) == 0:
        return False
    else:
        migrations = ingest_yaml_list(*migration_paths)

        munge_jobs = []
        for page in migrations:
            if 'sources' in page:
                migrations.extend(convert_multi_source(page))
                continue

            page = fix_migration_paths(page)
            fq_target, fq_source = resolve_page_path(page, conf)

            if page['source'].endswith('/'):
                migrations.extend(directory_expansion(fq_source, page, conf))
                continue

            page = trim_leading_slash_from_pages(page)
            prev = build_migration_task(fq_target, fq_source, app)

            if 'truncate' in page:
                build_truncate_task(page['truncate'], fq_target, fq_source, app)

            if 'transform' in page:
                prev.job = copy_always
                munge_jobs.append(build_transform_task(page['transform'], fq_target))

            if 'append' in page:
                prev.job = copy_always
                build_append_task(page, fq_target, migration_paths, app)

        post_process_tasks(app=app, tasks=munge_jobs)
        msg = 'added {0} migration jobs'.format(len(migrations))

        logger.info(msg)

        return True


########## Task Creators

def clean(conf, app):
    "Removes all migrated primer files according to the current spec."

    migration_paths = get_migration_specifications(conf)
    migrations = ingest_yaml_list(*migration_paths)

    targets = []
    for page in migrations:
        if 'sources' in page:
            migrations.extend(convert_multi_source(page))
            continue

        page = fix_migration_paths(page)

        targets.append(os.path.join(conf.paths.projectroot, conf.paths.source, page['target']))

    t = app.add('map')
    t.job = verbose_remove
    t.iter = targets
    t.description = 'clean primer migrations'

    logger.info('clean: removed {0} files'.format(len(targets)))

def build_migration_task(target, source, app):
    task = app.add('task')
    task.target = target
    task.job = copy_if_needed
    task.target = target
    task.dependency = source
    task.args = [ source, target, 'primer' ]

    return task

def build_transform_task(transform, target):
    return {
        'file': target,
        'type': 'primer-processing',
        'transform': transform
    }

def build_append_task(page, target, spec_files, app):
    task = app.add('task')
    task.target = page['target']
    task.dependency = spec_files
    task.job = append_to_file
    task.args = [ target, page['append'] ]

def build_truncate_task(truncate_spec, target, deps, app):
    task = app.add('task')
    task.target = target
    task.dependency = deps
    task.job = truncate_file
    task.args = {
        'fn': target,
        'start_after': truncate_spec['start-after'] if 'start-after' in truncate_spec else None,
        'end_before': truncate_spec['end-before'] if 'end-before' in truncate_spec else None
    }



import unittest
import logging
import shutil
import os

import polib

from giza.tools.serialization import ingest_yaml_doc
from giza.translate.translation import po_file_untranslated_to_text, extract_all_untranslated_po_entries, fill_po_file, write_po_files, auto_approve_po_entries
from giza.translate.utils import get_file_list

logger = logging.getLogger('test.test_translation')
TEST_PATH = os.path.abspath(os.path.join('..', os.path.dirname(__file__)))


class ExtractOnePoTestCase(unittest.TestCase):

    def setUp(self):
        os.makedirs(os.path.join(TEST_PATH, "temp_files"))

    def tearDown(self):
        logger.info('tear down')
        shutil.rmtree(os.path.join(TEST_PATH, "temp_files"), ignore_errors=True)

    def test_aggregation(self):
        with open(os.path.join(TEST_PATH, "temp_files", "source"),"w") as f:
            po_file_untranslated_to_text(f, os.path.join(TEST_PATH, "test_files", "docs", "aggregation.po"))
        with open(os.path.join(TEST_PATH, "temp_files", "source")) as f:
            self.assertEqual(f.read().strip(), "Aggregation\nA high-level introduction to aggregation.\nIntroduces the use and operation of the data aggregation modalities available in MongoDB.")

    def test_admin(self):
        with open(os.path.join(TEST_PATH, "temp_files", "source"),"w") as f:
            po_file_untranslated_to_text(f, os.path.join(TEST_PATH, "test_files", "docs", "administration.po"))
        with open(os.path.join(TEST_PATH, "temp_files", "source")) as f:
            self.assertEqual(f.read().strip(), "Administration")


class ExtractMultiplePoTestCase(unittest.TestCase):

    def setUp(self):
        os.makedirs(os.path.join(TEST_PATH,"temp_files"))

    def tearDown(self):
        logger.info('tear down')
        shutil.rmtree(os.path.join(TEST_PATH, "temp_files"), ignore_errors=True)

    def test_aggregation(self):
        po_file_list = get_file_list(os.path.join(TEST_PATH, "test_files", "docs"), ["po","pot"])
        po_file_list.sort()
        extract_all_untranslated_po_entries(po_file_list, os.path.join(TEST_PATH, "temp_files"))
        with open(os.path.join(TEST_PATH, "temp_files", "source")) as f:
            self.assertEqual(f.read().strip(), "Administration\nAggregation\nA high-level introduction to aggregation.\nIntroduces the use and operation of the data aggregation modalities available in MongoDB.")


class FillOnePoTestCase(unittest.TestCase):

    def setUp(self):
        os.makedirs(os.path.join(TEST_PATH,"temp_files"))
        shutil.copytree(os.path.join(TEST_PATH, "test_files", "docs"), os.path.join(TEST_PATH, "temp_files", "docs"))

    def tearDown(self):
        logger.info('tear down')
        shutil.rmtree(os.path.join(TEST_PATH,"temp_files"), ignore_errors=True)

    def test_administration(self):
        fill_po_file(os.path.join(TEST_PATH, "temp_files", "docs", "administration.po"),"1", 0)
        f1 = polib.pofile(os.path.join(TEST_PATH, "temp_files", "docs", "administration.po"))
        f2 = polib.pofile(os.path.join(TEST_PATH, "test_files", "filled_docs", "administration.po"))
        for l1, l2 in zip(f1, f2):
            self.assertEqual(l1, l2)

    def test_aggregation(self):
        fill_po_file(os.path.join(TEST_PATH, "temp_files", "docs", "aggregation.po"),"2\n3\n4", 1)
        f1 = polib.pofile(os.path.join(TEST_PATH, "temp_files", "docs", "aggregation.po"))
        f2 = polib.pofile(os.path.join(TEST_PATH, "test_files", "filled_docs", "aggregation.po"))
        for l1, l2 in zip(f1, f2):
            self.assertEqual(l1, l2)


class FillMultiplePoTestCase(unittest.TestCase):

    def setUp(self):
        os.makedirs(os.path.join(TEST_PATH,"temp_files"))
        shutil.copytree(os.path.join(TEST_PATH, "test_files", "docs"), os.path.join(TEST_PATH, "temp_files", "docs"))

    def tearDown(self):
        logger.info('tear down')
        shutil.rmtree(os.path.join(TEST_PATH,"temp_files"), ignore_errors=True)

    def test_files(self):
        po_file_list = get_file_list(os.path.join(TEST_PATH, "temp_files", "docs"),["po","pot"])
        po_file_list.sort()
        write_po_files(po_file_list,os.path.join(TEST_PATH, "test_files", "doc_filler.txt"))
        f1 = polib.pofile(os.path.join(TEST_PATH, "temp_files", "docs", "aggregation.po"))
        f2 = polib.pofile(os.path.join(TEST_PATH, "test_files", "filled_docs", "aggregation.po"))
        for l1, l2 in zip(f1, f2):
            self.assertEqual(l1, l2)
        f1 = polib.pofile(os.path.join(TEST_PATH, "temp_files", "docs", "administration.po"))
        f2 = polib.pofile(os.path.join(TEST_PATH, "test_files", "filled_docs", "administration.po"))
        for l1, l2 in zip(f1, f2):
            self.assertEqual(l1, l2)


class AutoApprovePoTestCase(unittest.TestCase):

    def setUp(self):
        os.makedirs(os.path.join(TEST_PATH,"temp_files"))
        shutil.copy(os.path.join(TEST_PATH, "test_files", "approve.pot"), os.path.join(TEST_PATH, "temp_files", "approve.pot"))

    def tearDown(self):
        logger.info('tear down')
        shutil.rmtree(os.path.join(TEST_PATH,"temp_files"), ignore_errors=True)

    def test_approve(self):
        auto_approve_po_entries(os.path.join(TEST_PATH, "temp_files", "approve.pot"))
        po_file = polib.pofile(os.path.join(TEST_PATH, "temp_files", "approve.pot"))
        self.assertEqual([entry.msgstr for entry in po_file.translated_entries()], [":hardlink:`MongoDB-manual.epub`"])



# Copyright 2014 MongoDB, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import logging
import os

logger = logging.getLogger('giza.config.git')

from giza.core.git import GitRepo
from giza.config.base import RecursiveConfigurationBase, ConfigurationBase

class GitConfigBase(RecursiveConfigurationBase):
    def __init__(self, obj, conf, repo=None):
        super(GitConfigBase, self).__init__(obj, conf)
        self.repo = repo

    @property
    def repo(self):
        return self._repo

    @repo.setter
    def repo(self, path=None):
        if isinstance(path, GitRepo):
            self._repo = path
        elif path is None:
            self._repo = GitRepo(self.conf.paths.projectroot)
        elif os.path.isdir(path):
            self._repo = GitRepo(path)
        else:
            self._repo = GitRepo(os.getcwd())


class GitConfig(GitConfigBase):
    @property
    def commit(self):
        c = self.repo.sha('HEAD')
        self.state['commit'] = c
        return c

    @property
    def branches(self):
        if 'branches' not in self.state:
            self.branches = None
        return self.state['branches']

    @branches.setter
    def branches(self, value):
        self.state['branches'] = GitBranchConfig(None, self.conf, self.repo)

    @property
    def remote(self):
        if 'remote' not in self.state:
            self.remote = None
        return self.state['remote']

    @remote.setter
    def remote(self, value):
        self.state['remote'] = GitRemoteConfig(value)

class GitBranchConfig(GitConfigBase):
    @property
    def current(self):
        if 'current' not in self.state:
            self.current = None

        return self.state['current']

    @current.setter
    def current(self, value):
        self.state['current'] = self.repo.current_branch()

    @property
    def manual(self):
        if 'manual' not in self.state:
            self.manual = None

        return self.state['manual']

    @manual.setter
    def manual(self, value):
        if 'git' in self.conf.runstate.branch_conf and 'branches' in self.conf.runstate.branch_conf['git']:
            if 'manual' in self.conf.runstate.branch_conf['git']['branches']:
                self.state['manual'] = self.conf.runstate.branch_conf['git']['branches']['manual']
            else:
                self.state['manual'] = None
        else:
            self.state['manual'] = None

    @property
    def published(self):
        if 'published' not in self.state:
            self.published = None

        return self.state['published']

    @published.setter
    def published(self, value):
        if 'git' in self.conf.runstate.branch_conf and 'branches' in self.conf.runstate.branch_conf['git']:
            if 'published' in self.conf.runstate.branch_conf['git']['branches']:
                p = self.conf.runstate.branch_conf['git']['branches']['published']

                if not isinstance(p, list):
                    msg = "published branches must be a list"
                    logger.critical(msg)
                    raise TypeError(msg)
                elif p[0] != 'master':
                    msg = "right now, we must publish master"
                    logger.critical(msg)
                    raise TypeError(msg)

                self.state['published'] = p

            else:
                self.state['published'] = []
        else:
            self.state['published'] = ['master']

class GitRemoteConfig(ConfigurationBase):
    _option_registry = ['upstream', 'tools']



import unittest
import logging
import shutil
import os

from giza.translate.utils import flip_text_direction, merge_files, get_file_list

logger = logging.getLogger('test.test_utils')
TEST_PATH = os.path.abspath(os.path.join('..', os.path.dirname(__file__)))


class ListFilesTestCase(unittest.TestCase):

    def test_simple(self):
        file_list = get_file_list(os.path.join(TEST_PATH, "test_files", "test_dir"), ['txt', 'yaml'])
        file_list.sort()
        self.assertEqual(file_list, [os.path.join(TEST_PATH, "test_files", "test_dir", "d1", "f4.txt"),
                                     os.path.join(TEST_PATH, "test_files", "test_dir", "f1.txt"),
                                     os.path.join(TEST_PATH, "test_files", "test_dir", "f2.txt"),
                                     os.path.join(TEST_PATH, "test_files", "test_dir", "f5.yaml")])

    def test_single(self):
        self.assertEqual(get_file_list(os.path.join(TEST_PATH, "test_files", "test_dir", "f1.txt"), ['txt']),
                         [os.path.join(TEST_PATH, "test_files", "test_dir", "f1.txt")])

    def test_bad_file(self):
        self.assertEqual(get_file_list(os.path.join(TEST_PATH, "temp_files", "mergdfadafadfaed.txt"),['txt']),[])


class MergeTestCase(unittest.TestCase):

    def setUp(self):
        os.makedirs(os.path.join(TEST_PATH, "temp_files"))

    def tearDown(self):
        logger.info('tear down')
        shutil.rmtree(os.path.join(TEST_PATH, "temp_files"), ignore_errors=True)

    def test_simple(self):
        merge_files(os.path.join(TEST_PATH, "test_files", "merged.txt"),
                   [os.path.join(TEST_PATH, "test_files", "f1"),
                    os.path.join(TEST_PATH, "test_files", "f2"),
                    os.path.join(TEST_PATH, "test_files", "f3")], ["+ ","- ", "~ "])
        with open(os.path.join(TEST_PATH, "test_files", "merged.txt")) as f:
            self.assertEqual(f.read().strip(), "+ 1a\n- 2a\n~ 3a\n\n+ 1b\n- 2b\n~ 3b\n\n+ \n- 2c\n~ 3c\n\n+ 1c\n- 2d")

    def test_empty(self):
        merge_files(os.path.join(TEST_PATH, "temp_files", "merged.txt"),
                   [os.path.join(TEST_PATH, "test_files", "empty.txt")], ["+ "])
        with open(os.path.join(TEST_PATH, "temp_files", "merged.txt")) as f:
            self.assertEqual(f.read().strip(), "")

    def test_annotation_failure(self):
        with self.assertRaises(Exception):
            merge_files(os.path.join(TEST_PATH, "temp_files", "merged.txt"),
                       [os.path.join(TEST_PATH, "test_files", "f1"),
                        os.path.join(TEST_PATH, "test_files", "f2")], ["+ "])

    def test_bad_file(self):
        with self.assertRaises(Exception):
            merge_files(os.path.join(TEST_PATH, "temp_files", "merged.txt"),
                       [os.path.join(TEST_PATH, "test_files", "fdafda1")], ["+ "])

class FlipTextTestCase(unittest.TestCase):

    def setUp(self):
        os.makedirs(os.path.join(TEST_PATH, "temp_files"))

    def tearDown(self):
        logger.info('tear down')
        shutil.rmtree(os.path.join(TEST_PATH, "temp_files"), ignore_errors=True)

    def test_simple(self):
        flip_text_direction(os.path.join(TEST_PATH, "test_files", "flip_text.txt"),
                            os.path.join(TEST_PATH, "temp_files", "flipped.txt"))
        with open(os.path.join(TEST_PATH, "temp_files", "flipped.txt")) as f:
            self.assertEqual(f.read().strip(), "haduj ,olleh\nuoy era woh ,ih\n\na\ncb\ndoog")

    def test_empty(self):
        flip_text_direction(os.path.join(TEST_PATH, "test_files", "empty.txt"),
                            os.path.join(TEST_PATH, "temp_files", "flipped.txt"))
        with open(os.path.join(TEST_PATH, "temp_files", "flipped.txt")) as f:
            self.assertEqual(f.read().strip(), "")

    def test_bad_file(self):
        with self.assertRaises(Exception):
            flip_text_direction(os.path.join(TEST_PATH, "test_files", "empdfadfadty.txt"),
                                os.path.join(TEST_PATH, "temp_files", "flipped.txt"))




# Copyright 2014 MongoDB, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
Defines the ways that example content can inherit content from other
examples. This module holds two classes: a representation of a single file that
contains a set of examples and

See :mod:`giza.core.inheritance`
"""

import logging
import os.path

logger = logging.getLogger('giza.content.examples.inheritance')

from giza.config.base import RecursiveConfigurationBase
from giza.content.examples.models import ExampleData, ExampleCase
from giza.content.helper import edition_check
from giza.core.inheritance import InheritableContentError, DataContentBase, DataCache
from giza.tools.serialization import ingest_yaml_list

# Example specific inheritance machinery

class ExampleError(Exception): pass

class ExampleFile(DataContentBase):
    """
    There is a one to one mapping of example files and output examples. Each
    example file has some "starting data," or a "collection" and then a sequence
    of operation and result pairs (i.e. "examples") that contain both a sequence
    of operations *and* an expected result. Each example *must* have a
    collection, and typically has 1 or more output examples.

    The ``ingest()`` method adds a check to ensure to produce an error to ensure
    that there's a "collection" value, while ``add()`` and ``fetch()`` capture
    collection and examples separately.
    """

    def ingest(self, src):
        super(ExampleFile, self).ingest(src)

        #if self.collection is not self.collection:
        #    logger.error(self.state)
        #    m = 'all examples must have a collection'
        #    logger.error(m)
        #    raise InheritableContentError(m)

    @property
    def collection(self):
        if 'collection' not in self.state:
            return None
        else:
            return self.content[self.state['collection']]

    @collection.setter
    def collection(self, value):
        if 'collection' in self.state:
            m = 'example spec "{0}" already exists'.format(self.collection.name)
            logger.error(m)
            raise ExampleError(m)
        elif isinstance(value, ExampleData):
            self.content[value.name] = value
            self.state['collection'] = value.name
        else:
            raise TypeError

    @property
    def examples(self):
        return [ example
                 for example in self.content.values()
                 if 'collection' not in example
        ]

    def add(self, doc):
        if edition_check(doc, self.conf) is False:
            return

        if 'collection' in doc:
            self.collection = ExampleData(doc, self.conf)

            if not self.collection.is_resolved():
                self.collection.resolve(self.data)
        elif 'operation' in doc:
            op = ExampleCase(doc, self.conf)
            if op.ref in self.content:
                m = 'example named {0} already exists'.format(op.ref)
                logger.error(m)
                raise ExampleError(m)
            else:
                self.content[op.ref] = op
                if not op.is_resolved():
                    op.resolve(self.data)
                logger.debug('added operation {0}'.format(op.name))

    def fetch(self, ref):
        if ref == self.collection.name:
            if not self.collection.is_resolved():
                self.collection.resolve(self.data)

            return self.collection

        elif ref in self.content:
            content = self.content[ref]

            if not content.is_resolved():
                content.resolve(self.data)

            return content

        else:
            m = 'content with ref "{0}" not found'.format(ref)
            logger.error(m)
            raise ExampleError(m)

class ExampleDataCache(DataCache):
    content_class = ExampleFile



# Copyright 2014 MongoDB, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import logging

logger = logging.getLogger('giza.content.release.views')

from rstcloth.rstcloth import RstCloth
from giza.content.steps.views import render_action

def render_releases(release, conf):
    r = RstCloth()

    release.replacement = {
        'version': conf.version.release,
        'branch': conf.version.branch,
        'stable': conf.version.stable,
    }

    release.render() # run replacements

    render_action(release, indent=0, level=2, r=r)

    return r



# Copyright 2014 MongoDB, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
Glues the reading and processing (:mod:`giza.content.examples.inheritance`) of
the example data (:mod:`giza.content.examples.models`) to the generation of the
output format (:mod:`giza.content.examples.views`).
"""

import logging
import os

logger = logging.getLogger('giza.content.examples')

from giza.tools.files import expand_tree, safe_create_directory, verbose_remove
from giza.content.examples.inheritance import ExampleDataCache
from giza.content.examples.views import full_example

def write_full_example(collection, examples, fn):
    content = full_example(collection, examples)
    content.write(fn)

def example_tasks(conf, app):
    # In the beginning of this operation, which executes in the main thread, we
    # read all files in "source/includes/" and sub-directories hat start with
    # "example-*"

    include_dir = os.path.join(conf.paths.projectroot, conf.paths.branch_includes)
    fn_prefix = os.path.join(include_dir, 'example')

    example_sources = [ fn for fn in
                        expand_tree(include_dir, 'yaml')
                        if fn.startswith(fn_prefix) ]

    # process the corpus of example data.
    d = ExampleDataCache(example_sources, conf)

    if len(d) > 0:
        safe_create_directory(fn_prefix)

    for fn, exmpf in d.file_iter():
        basename = fn[len(fn_prefix)+1:-5]

        out_fn = os.path.join(conf.paths.projectroot,
                              conf.paths.branch_includes, 'examples', basename) + '.rst'

        t = app.add('task')
        t.target = out_fn
        t.dependency = fn
        t.job = write_full_example
        t.args = (exmpf.collection, exmpf.examples, out_fn)
        t.description = 'generate an example for ' + basename

    logger.debug('added all tasks for example generation')

def example_clean(conf, app):
    fn_prefix = os.path.join(include_dir, 'example')

    example_sources = [ fn for fn in
                        expand_tree(include_dir, 'yaml')
                        if fn.startswith(fn_prefix) ]

    for fn in example_sources:
        basename = fn[len(fn_prefix)+1:-5]

        out_fn = os.path.join(conf.paths.projectroot,
                              conf.paths.branch_includes, 'examples', basename) + '.rst'

        t = app.add('task')
        t.target = True
        t.dependency = out_fn
        t.job = verbose_remove
        t.args = [out_fn]



# Copyright 2014 MongoDB, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import time
import os
import subprocess
import logging
from tempfile import NamedTemporaryFile

logger = logging.getLogger('giza.command')

class CommandError(Exception): pass

class DevNull(object):
    name = os.devnull

class CommandResult(object):
    def __init__(self, cmd=None, err=None, out=None, return_code=None):
        self._cmd = cmd
        self._err = err
        self._out = out
        self._return_code = return_code
        self._captured = ''

    @property
    def succeeded(self):
        return True if self.return_code == 0 else False

    @property
    def failed(self):
        return False if self.return_code == 0 else True

    @property
    def cmd(self):
        return self._cmd

    @cmd.setter
    def cmd(self, value):
        self._cmd = value

    @property
    def err(self):
        return self._err

    @err.setter
    def err(self, value):
        self._err = value

    @property
    def out(self):
        return self._out

    @out.setter
    def out(self, value):
        self._out = value

    @property
    def return_code(self):
        return self._return_code

    @return_code.setter
    def return_code(self, value):
        if not isinstance(value, int):
            raise CommandError('invalid return code type')
        else:
            self._return_code = value

    @property
    def captured(self):
        return self._captured

    @captured.setter
    def captured(self, value):
        self._captured = value


def command(command, capture=True, ignore=False, logger=None):
    """
    Inspired by Fabric's ``local()`` operation. Runs a shell command, optionally
    captures the output, and returns a :class:`~giza.command.CommandResult` object.
    """

    if isinstance(command, (list, tuple)):
        command = ' '.join(command)

    if logger is None:
        log_output = False
        logger = logging.getLogger('giza.command')
    else:
        log_output = True
        logger.info(command)

    logger.debug("running '{0}'".format(command))
    if capture is False:
        tmp_out = DevNull()
        tmp_err = DevNull()
    else:
        tmp_out = NamedTemporaryFile()
        tmp_err = NamedTemporaryFile()

    with open(tmp_out.name, 'w') as tout:
        with open(tmp_err.name, 'w') as terr:
            p = subprocess.Popen(command, stdout=tout, stderr=terr, shell=True)

            while True:
                if p.poll() is not None:
                    break

    if capture is False:
        stdout = ""
        stderr = ""
    else:
        with open(tmp_out.name, 'r') as f:
            stdout = ''.join(f.readlines()).strip()
        with open(tmp_err.name, 'r') as f:
            stderr = ''.join(f.readlines()).strip()

    out = CommandResult(cmd=command,
                        err=stderr,
                        out=stdout,
                        return_code=p.returncode)
    out.captured = capture

    if log_output is True and capture is True:
        if len(stdout) > 0:
            logger.info(stdout)
        if len(stderr) > 0:
            logger.error(stderr)

    if out.succeeded is True or ignore is True:
        return out
    else:
        raise CommandError('"{0}" returned code {1}'.format(out.cmd, out.return_code))

def verbose_command(cmd, capture=False, ignore=False):
    """
    .. deprecated:: 0.2.7
    """

    if isinstance(cmd, dict):
        cmd = cmd['command']

    if isinstance(cmd, list):
        cmd_str = ' '.join(cmd)
    else:
        cmd_str = cmd

    logger.warning('verbose command is deprecated: ({0})'.format(cmd_str))

    logger.info("running command: " + cmd_str)
    command(cmd_str, capture, ignore)



# Copyright 2014 MongoDB, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import collections
import logging
import re

logger = logging.getLogger('giza.transformation')

from giza.tools.files import copy_always, copy_if_needed, encode_lines_to_file, decode_lines_from_file
from giza.tools.serialization import ingest_yaml

class ProcessingError(Exception):
    pass

def munge_page(fn, regex, out_fn=None,  tag='build'):
    if out_fn is None:
        out_fn = fn

    page_lines = [ munge_content(ln, regex) for ln in decode_lines_from_file(fn)
                   if ln is not None ]

    if len(page_lines) > 0:
        encode_lines_to_file(out_fn, page_lines)
        logger.info('{0}: processed {1}'.format(tag, fn))
    else:
        logger.warning('{0}: did not write {1}'.format(tag, out_fn))

def munge_content(content, regex):
    if isinstance(regex, list):
        for cregex, subst in regex:
            content = cregex.sub(subst, content)
        return content
    else:
        return regex[0].sub(regex[1], content)


def truncate_file(fn, start_after=None, end_before=None):
    with open(fn, 'r') as f:
        source_lines = f.readlines()

    start_idx = 0
    end_idx = len(source_lines) - 1

    for idx, ln in enumerate(source_lines):
        if start_after is not None:
            if start_idx == 0 and ln.startswith(start_after):
                start_idx = idx - 1
                start_after = None

        if end_before is not None:
            if ln.startswith(end_before):
                end_idx = idx
                break

    with open(fn, 'w') as f:
        f.writelines(source_lines[start_idx:end_idx])

def append_to_file(fn, text):
    with open(fn, 'a') as f:
        f.write('\n')
        f.write(text)


def process_page(fn, output_fn, regex, app, builder='processor', copy='always'):
    t = app.add('task')
    t.job = _process_page
    t.args = [fn, output_fn, regex, copy, builder ]
    t.target = output_fn
    t.depenency = None
    t.description = "modify page"

    logger.debug('added tasks to process file: {0}'.format(fn))

def _process_page(fn, output_fn, regex, copy, builder):
    tmp_fn = fn + '~'

    munge_page(fn=fn, out_fn=tmp_fn, regex=regex)

    cp_args = dict(source_file=tmp_fn,
                   target_file=output_fn,
                   name=builder)

    if copy == 'always':
        copy_always(**cp_args)
    else:
        copy_if_needed(**cp_args)

def post_process_tasks(app, tasks=None, source_fn=None):
    """
    input documents should be:

    {
      'transform': {
                     'regex': str,
                     'replace': str
                   }
      'type': <str>
      'file': <str|list>
    }

    ``transform`` can be either a document or a list of documents.
    """

    if tasks is None:
        if source_fn is not None:
            tasks = ingest_yaml(source_fn)
        else:
            raise ProcessingError('[ERROR]: no input tasks or file')
    elif not isinstance(tasks, collections.Iterable):
        raise ProcessingError('[ERROR]: cannot parse post processing specification.')

    def rjob(fn, regex, type):
        page_app = app.add('app')
        process_page(fn=fn, output_fn=fn, regex=regex, app=page_app, builder=type)

    for job in tasks:
        if not isinstance(job, dict):
            raise ProcessingError('[ERROR]: invalid replacement specification.')
        elif not 'file' in job and not 'transform' in job:
            raise ProcessingError('[ERROR]: replacement specification incomplete.')

        if 'type' not in job:
            job['type'] = 'processor'

        if isinstance(job['transform'], list):
            regex = [ (re.compile(rs['regex']), rs['replace'])
                      for rs in job['transform'] ]
        else:
            regex = (re.compile(job['transform']['regex']), job['transform']['replace'])

        if not isinstance(job['file'], list):
            job['file'] = [ job['file'] ]

        for fn in job['file']:
            rjob(fn, regex, job['type'])



# Copyright 2014 MongoDB, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import logging
import os

logger = logging.getLogger('giza.config.corpora')

from giza.config.base import ConfigurationBase

class SourceConfig(ConfigurationBase):
    _option_registry = ['name', 'source_file_path', 'target_file_path',
                        'percent_train', 'percent_tune', 'percent_test',
                        'percent_of_train', 'percent_of_tune',
                        'percent_of_test', 'length', 'end']

class CorporaConfig(ConfigurationBase):
    _option_registry = ['container_path', 'source_language', 'target_language',
                        'sources']

    def __init__(self, input_obj):
        input_obj = self.transform(input_obj)

        super(CorporaConfig, self).__init__(input_obj)

        self.verify_percentages()
        self.get_file_lengths()

    def verify_percentages(self):
        '''This function verifies that the percentages are valid and add up to 100
        '''
        for file_name, source in self.sources.items():
            if source.percent_train + source.percent_tune + source.percent_test != 100:
                error = "Source percentages don't add up to 100 for " + file_name
                logger.error(error)
                raise TypeError(error)

        for t in ('train', 'tune', 'test'):
            tot = 0
            for file_name,source in self.sources.items():
                tot += source.state['percent_of_' + t]
            if tot != 100:
                error = "Contribution percentages don't add up to 100 for " + t
                logger.error(error)
                raise TypeError(error)

    def get_file_lengths(self):
        '''This function adds the file lengths of the files to the configuration dictionary
        '''
        for file_name, source in self.sources.items():
            with open(source.source_file_path, 'r') as f:
                length1 = len(f.readlines())
            with open(source.target_file_path, 'r') as f:
                length2 = len(f.readlines())
            if length1 != length2:
                error = "Lengths of files for "+file_name+" are not identical"
                logger.error(error)
                raise TypeError(error)
            source.length = length1

    def dict(self):
        '''This function overrides the default dict() function to transform a corpora object into a dictionary
        '''
        # use listcomp in sources for py3 compatibility
        d = {
            'container_path': self.container_path,
            'target_language': self.target_language,
            'sources': [ s for s in self.sources.values() ]
        }

        return d

    def transform(self, input_obj):
        '''This function takes a configuration file object cand creates a dictionary of the useful information.
        It also sets defaults for certain arguments so they do not need to be specified by the user for it to work

        :param dict input_obj: original input object

        :returns: a processed dictionary of the configuration
        '''
        d = {
            'container_path' : os.path.expanduser(input_obj['container_path']),
            'source_language': input_obj['source_language'],
            'target_language': input_obj['target_language'],
            'sources': {}
        }

        # handles sources section of config
        for source in input_obj['sources']:
            for t in ('train', 'tune', 'test'):
                if source['percent_' + t] < 0 or source['percent_' + t] > 100:
                    logger.error("Invalid percentage")
                    raise TypeError("Invalid percentage")
            source['source_file_path'] = os.path.expanduser(source['source_file_path'])
            source['target_file_path'] = os.path.expanduser(source['target_file_path'])
            s = SourceConfig(source)
            s.percent_of_train = 0
            s.percent_of_tune = 0
            s.percent_of_test = 0
            s.end = 0
            d['sources'][source['name']] = s

        # handles source contributions section of config
        for t in ('train','tune','test'):
            for source in input_obj['source_contributions'][t]:
                if source['percent_of_corpus'] < 0 or source['percent_of_corpus'] > 100:
                    logger.error("Invalid percentage")
                    raise TypeError("Invalid percentage")
                d['sources'][source['name']].state['percent_of_' + t] = source['percent_of_corpus']

        return d




# Copyright 2014 MongoDB, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os
import logging

logger = logging.getLogger('giza.content.options.tasks')

from giza.tools.files import expand_tree, verbose_remove
from giza.tools.strings import hyph_concat
from giza.content.options.inheritance import OptionDataCache
from giza.content.options.views import render_options

def get_option_fn_prefix(conf):
    return os.path.join(conf.paths.projectroot, conf.paths.branch_includes, 'option')

def option_outputs(conf):
    include_dir = os.path.join(conf.paths.projectroot, conf.paths.branch_includes)

    fn_prefix = get_option_fn_prefix(conf)

    return [ fn for fn in
             expand_tree(include_dir, 'yaml')
             if fn.startswith(fn_prefix) ]

def write_options(option, fn, conf):
    content = render_options(option, conf)
    content.write(fn)
    logger.info(fn)

def option_tasks(conf, app):
    fn_prefix = get_option_fn_prefix(conf)
    option_sources = option_outputs(conf)
    o = OptionDataCache(option_sources, conf)

    if len(option_sources) and not os.path.isdir(fn_prefix):
        os.makedirs(fn_prefix)

    for dep_fn, option in o.content_iter():
        if option.program.startswith('_'):
            continue

        out_fn = hyph_concat(option.directive, option.program, option.name) + '.rst'
        output_fn = os.path.join(fn_prefix, out_fn)

        t = app.add('task')
        t.target = output_fn
        t.dependency = [dep_fn]
        t.job = write_options
        t.args = (option, output_fn, conf)
        t.description = 'generating option file "{0}" from "{1}"'.format(output_fn, dep_fn)

def option_clean(conf, app):
    for fn in option_outputs(conf):
        task = app.add('task')
        task.job = verbose_remove
        task.args = [fn]
        task.description = 'removing {0}'.format(fn)



# Copyright 2014 MongoDB, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
Given a a specification of an image, and an ``svg`` in the configured image
directory, build desired image artifacts (using ``inkscape``) as well as an
``rst`` file that includes all of content to include the image in the
output.

With images generated from SVG, we separate the generation of build artifacts
(images) from their source, which means the documentation can use
output-specific images and potentially translate the text in the diagrams. By
generating all of the ``rst`` to inclde the images, its possible display and
maintain the presentation of images systematically and centrally.

The format of the image specification is: ::

  {
    "name": <imgBaseName>,
    "alt": <text>,
    "output": [
      {
        "type": <string>,
        "tag": <string>,
        "dpi": <int>,
        "width": <int>
      },
    ]
  }

The ``tag`` field is optional and appends a string to the generated file name,
to facilitate multiple output targets without having conflicting file names.

The ``<imgBaseName>`` is the name of the ``.svg`` file without the
extension. The main image-generation operation is in
:func:`giza.content.images.image_tasks()`, while definition of the `rst`` content
is in :func:`giza.content.images.generate_image_pages()`.

.. note::

   The current implementation does not strictly enforce the metadata schema.
"""

import sys
import os.path
import logging

logger = logging.getLogger('giza.content.images')

from docutils.core import publish_parts
from rstcloth.rstcloth import RstCloth

from giza.tools.command import command
from giza.tools.files import verbose_remove
from giza.tools.serialization import ingest_yaml_list
from giza.tools.strings import dot_concat, hyph_concat

## Internal Supporting Methods

def generate_image_pages(dir, name, alt, output, conf):
    r = RstCloth()

    image = '/'.join([dir, name])

    for img_output in output:
        img_output['width'] = str(img_output['width']) + 'px'

        if img_output['type'] == 'offset':
            build_type = 'eps'
        else:
            build_type = 'png'

        r.newline()

        if 'tag' in img_output:
            tag = ''.join(['-', img_output['tag'], '.', build_type ])
        else:
            tag = '.' + build_type

        options = [('alt', alt), ('align', 'center'), ('figwidth', img_output['width'])]

        if 'scale' in img_output:
            options.append(('scale', img_output['scale']))
        if img_output['type'] in 'print':
            r.directive('only', 'latex and not offset', wrap=False)
            r.newline()

            r.directive(name='figure',
                        arg='/images/{0}{1}'.format(name, tag),
                        fields=options,
                        indent=3)
            r.newline()
        elif img_output['type'] == 'offset':
            tex_figure = [
                r'\begin{figure}[h!]',
                   r'\centering',
                   ''.join([r'\includegraphics[width=', img_output['width'],
                            ']{', name, tag, '}' ]),
                r'\end{figure}'
            ]

            r.directive('only', 'latex and offset', wrap=False)
            r.newline()
            r.directive('raw', 'latex', content=tex_figure, indent=3)
        else:
            r.directive('only', 'website and slides', wrap=False)
            r.newline()
            r.directive(name='figure',
                        arg='/images/{0}{1}'.format(name, tag),
                        fields=options,
                        indent=3)

            r.newline()

            r.directive('only', 'website and html', wrap=False)
            r.newline()
            r.directive(name='figure',
                        arg='/images/{0}{1}'.format(name, tag),
                        fields=options,
                        indent=3)

            r.newline()

            if img_output['width'] > 740:
                options[2] = ('figwidth', 740)

            r.directive('only', 'website and not (html or slides)', wrap=False)
            r.newline()
            img_str = ''.join(['<div class="figure align-center" style="max-width:{5};">',
                               '<img src="{0}/{1}/_images/{2}{3}" alt="{4}">', '</img>',
                               '{6}</div>' ])
            alt_html = publish_parts(alt, writer_name='html')['body'].strip()
            r.directive(name='raw', arg='html',
                        content=img_str.format(conf.project.url,
                                               conf.git.branches.current, name, tag, alt,
                                               img_output['width'], alt_html),
                        indent=3)

        r.newline()

    image_rst_file_path = os.path.join(conf.paths.projectroot, image + '.rst')
    r.write(image_rst_file_path)
    logger.debug('generated include file {0}.rst'.format(image))

def _get_inkscape_cmd():
    if sys.platform in ['linux', 'linux2']:
        return '/usr/bin/inkscape'
    elif sys.platform == 'darwin':
        inkscape = '/Applications/Inkscape.app/Contents/Resources/bin/inkscape'
        if os.path.exists(inkscape):
            return inkscape

    return 'inkscape'

def _generate_images(cmd, dpi, width, target, source):
    full_cmd = cmd.format(cmd=_get_inkscape_cmd(),
                          dpi=dpi,
                          width=width,
                          target=target,
                          source=source)
    command(full_cmd)

    logger.debug(full_cmd)

    logger.info('generated image file {0}'.format(target))

def get_images_metadata_file(conf):
    base = None
    for fn in conf.system.files.paths:
        if isinstance(fn, dict):
            if 'images' in fn:
                base = fn['images']
                break
        else:
            if fn.startswith('images'):
                base = fn
                break

    if base is None:
        return None
    elif base.startswith('/'):
        base = base[1:]
        return os.path.join(conf.paths.projectroot, base)
    else:
        return os.path.join(conf.paths.projectroot, conf.paths.builddata, base)

def image_tasks(conf, app):
    meta_file = get_images_metadata_file(conf)

    if 'images' not in conf.system.files.data:
        logger.info('no images to generate')
        return

    if isinstance(conf.system.files.data.images, list):
        images = conf.system.files.data.images
    else:
        images = [ conf.system.files.data.images ]

    image_dir = conf.paths.branch_images

    for image in images:
        image['dir'] = image_dir
        image['conf'] = conf

        source_base = os.path.join(conf.paths.projectroot, image['dir'], image['name'])
        source_file = dot_concat(source_base, 'svg')
        source_core = os.path.join(conf.paths.projectroot, conf.paths.images, image['name'] + '.svg' )
        rst_file = dot_concat(source_base, 'rst')

        if not os.path.isfile(source_core):
            logger.error('"{0}" does not exist'.format(source_core))
            continue

        t = app.add('task')
        t.conf = conf
        t.job = generate_image_pages
        t.args = image # as kwargs
        t.description = "generating rst include file {0} for {1}".format(rst_file, source_core)
        t.target = rst_file
        t.dependency = [meta_file, os.path.abspath(__file__) ]
        logger.debug('adding task for image rst file: {0}'.format(rst_file))

        for output in image['output']:
            if output['type'] == 'offset':
                build_type = 'eps'
            else:
                build_type = 'png'

            if 'tag' in output:
                tag = '-' + output['tag']
            else:
                tag = ''

            target_img = ''.join([source_base, tag, '.', build_type])

            if build_type == 'png':
                inkscape_cmd = '{cmd} -z -d {dpi} -w {width} -y 0.0 -e >/dev/null {target} {source}'
            elif build_type == 'eps':
                inkscape_cmd = '{cmd} -z -d {dpi} -w {width} -y 1.0 -E >/dev/null {target} {source}'

            t = app.add('task')
            t.conf = conf
            t.job = _generate_images
            t.args = [ inkscape_cmd, output['dpi'], output['width'], target_img, source_file ]
            t.target = target_img
            t.dependency = [ source_core ]
            t.description = 'generating image file {0} from {1}'.format(target_img, source_core)
            logger.debug('adding image creation job for {0}'.format(target_img))

def image_clean(conf, app):
    if 'images' not in conf.system.files.data:
        logger.info('no images to clean')
        return

    for image in conf.system.files.data.images:
        source_base = os.path.join(conf.paths.projectroot, conf.paths.images, image['name'])

        rm_rst = app.add('task')
        rm_rst.job = verbose_remove
        rm_rst.args = dot_concat(source_base, 'rst')

        for output in image['output']:
            rm_tag_image = app.add('task')
            rm_tag_image.job = verbose_remove
            if 'tag' in output:
                rm_tag_image.args = dot_concat(hyph_concat(source_base, output['tag']), 'png')
            else:
                rm_tag_image.args = dot_concat(source_base, 'png')



# -*- coding: utf-8 -*-
#
# MongoDB documentation build configuration file, created by
# sphinx-quickstart on Mon Oct  3 09:58:40 2011.
#
# This file is execfile()d with the current directory set to its containing dir.

import sys
import os
import datetime
import logging

from giza.config.runtime import RuntimeStateConfig
from giza.config.helper import fetch_config, get_versions, get_manual_path
from giza.tools.strings import dot_concat

logging.basicConfig(level=logging.INFO)
conf = fetch_config(RuntimeStateConfig())
sconf = conf.system.files.data.sphinx_local

sys.path.append(os.path.join(conf.paths.projectroot, conf.paths.buildsystem, 'sphinxext'))

# -- General configuration ----------------------------------------------------

needs_sphinx = '1.0'

extensions = [
    'sphinx.ext.extlinks',
    'sphinx.ext.todo',
    'mongodb',
    'directives',
    'intermanual',
]

locale_dirs = [ os.path.join(conf.paths.projectroot, conf.paths.locale) ]
gettext_compact = False

templates_path = ['.templates']
exclude_patterns = []

source_suffix = '.txt'

master_doc = sconf.master_doc
language = 'en'
project = sconf.project
copyright = u'2011-{0}'.format(datetime.date.today().year)
version = conf.version.branch
release = conf.version.release

rst_epilog = '\n'.join([
    '.. include:: {0}/hash.rst'.format(conf.paths.includes[len(conf.paths.source):]),
    '.. |copy| unicode:: U+000A9',
])

pygments_style = 'sphinx'

extlinks = {
    'issue': ('https://jira.mongodb.org/browse/%s', '' ),
    'api': ('http://api.mongodb.org/%s', ''),
    'source': ('https://github.com/mongodb/mongo/blob/master/%s', ''),
    'docsgithub' : ( 'http://github.com/mongodb/docs/blob/{0}/%s'.format(conf.git.branches.current), ''),
    'hardlink' : ( 'http://docs.mongodb.org/{0}/%s'.format(conf.git.branches.current), ''),
    'manual': ('http://docs.mongodb.org/manual%s', ''),
    'ecosystem': ('http://docs.mongodb.org/ecosystem%s', ''),
    'meta-driver': ('http://docs.mongodb.org/meta-driver/latest%s', ''),
    'mms': ('https://mms.mongodb.com/help%s', ''),
    'mms-hosted': ('https://mms.mongodb.org/help-hosted%s', ''),
    'about': ('http://www.mongodb.org/about%s', '')
}

## add `extlinks` for each published version.
for i in conf.git.branches.published:
    extlinks[i] = ( ''.join([ conf.project.url, '/', i, '%s' ]), '' )

intersphinx_mapping = {}
for i in conf.system.files.data.intersphinx:
    intersphinx_mapping[i.name] = ( i.url, os.path.join(conf.paths.projectroot,
                                                        conf.paths.output,
                                                        i.path))

languages = [
    ("ar", "Arabic"),
    ("cn", "Chinese"),
    ("cs", "Czech"),
    ("de", "German"),
    ("es", "Spanish"),
    ("fr", "French"),
    ("hu", "Hungarian"),
    ("id", "Indonesian"),
    ("it", "Italian"),
    ("jp", "Japanese"),
    ("ko", "Korean"),
    ("lt", "Lithuanian"),
    ("pl", "Polish"),
    ("pt", "Portuguese"),
    ("ro", "Romanian"),
    ("ru", "Russian"),
    ("tr", "Turkish"),
    ("uk", "Ukrainian")
]

# -- Options for HTML output ---------------------------------------------------

html_theme = sconf.theme.name
html_theme_path = [ os.path.join(conf.paths.buildsystem, 'themes') ]
html_title = conf.project.title
htmlhelp_basename = 'MongoDB'

html_logo = sconf.logo
html_static_path = sconf.paths.static

html_copy_source = False
html_use_smartypants = True
html_domain_indices = True
html_use_index = True
html_split_index = False
html_show_sourcelink = False
html_show_sphinx = True
html_show_copyright = True

manual_edition_path = '/.'.join([conf.project.url,
                                 conf.git.branches.current,
                                 sconf.theme.book_path_base])

html_theme_options = {
    'branch': conf.git.branches.current,
    'translations': languages,
    'language': language,
    'repo_name': sconf.theme.repo,
    'jira_project': sconf.theme.jira,
    'google_analytics': sconf.theme.google_analytics,
    'project': sconf.theme.project,
    'version': version,
    'sitename': sconf.theme.sitename,
    'nav_excluded': sconf.theme.nav_excluded,
}

html_sidebars = sconf.sidebars

# -- Options for LaTeX output --------------------------------------------------

latex_documents = []
if 'pdfs' in conf.system.files.data:
    for pdf in conf.system.files.data.pdfs:
        latex_documents.append((pdf.source, pdf.output, pdf.title, pdf.author, pdf.doc_class))

latex_preamble_elements = [ r'\DeclareUnicodeCharacter{FF04}{\$}',
                            r'\DeclareUnicodeCharacter{FF0E}{.}',
                            r'\PassOptionsToPackage{hyphens}{url}',
                            r'\usepackage{upquote}',
                            r'\pagestyle{plain}',
                            r'\pagenumbering{arabic}' ]
latex_elements = {
    'preamble': '\n'.join(latex_preamble_elements),
    'pointsize': '10pt',
    'papersize': 'letterpaper'
}

latex_paper_size = 'letter'
latex_use_parts = False
latex_show_pagerefs = True
latex_show_urls = 'footnote'
latex_domain_indices = False
latex_logo = None
latex_appendices = []

# -- Options for manual page output --------------------------------------------

man_pages = []
if 'manpages' in conf.system.files.data:
    for mp in conf.system.files.data.manpages:
        man_pages.apend((mp.file, mp.name, mp.title, mp.authors, mp.section))

# -- Options for Epub output ---------------------------------------------------

# Bibliographic Dublin Core info.
epub_title = conf.project.title
epub_author = u'MongoDB Documentation Project'
epub_publisher = u'MongoDB, Inc.'
epub_copyright = copyright
epub_theme = 'epub_mongodb'
epub_tocdup = True
epub_tocdepth = 3
epub_language = language
epub_scheme = 'url'
epub_identifier = '/'.join([conf.project.url, conf.git.branches.current])
epub_exclude_files = []

epub_pre_files = []
epub_post_files = []

# put it into your conf.py
def setup(app):
    # disable versioning for speed
    from sphinx.builders.gettext import I18nBuilder
    I18nBuilder.versioning_method = 'none'

    def doctree_read(app, doctree):
        if not isinstance(app.builder, I18nBuilder):
            return
        from docutils import nodes
        from sphinx.versioning import add_uids
        list(add_uids(doctree, nodes.TextElement))
    app.connect('doctree-read', doctree_read)



import unittest
import logging
import shutil
import os

from giza.tools.serialization import ingest_yaml_doc
from giza.config.corpora import CorporaConfig
from giza.translate.corpora import create_hybrid_corpora, create_corpus_from_po, create_corpus_from_dictionary

logger = logging.getLogger('test.test_corpora')
TEST_PATH = os.path.abspath(os.path.join('..', os.path.dirname(__file__)))


class DictToCorpusTestCase(unittest.TestCase):

    def setUp(self):
        os.makedirs(os.path.join(TEST_PATH, "temp_files"))

    def tearDown(self):
        logger.info('tear down')
        shutil.rmtree(os.path.join(TEST_PATH, "temp_files"), ignore_errors=True)

    def test_english(self):
        create_corpus_from_dictionary(os.path.join(TEST_PATH, "test_files", "dict.txt"), os.path.join(TEST_PATH, "temp_files", "source.txt"), os.path.join(TEST_PATH,"temp_files", "target.txt"))
        with open(os.path.join(TEST_PATH,"temp_files","source.txt")) as f:
            self.assertEqual(f.read().strip(), "April\nApr\nAugust\nAug")

    def test_spanish(self):
        create_corpus_from_dictionary(os.path.join(TEST_PATH,"test_files", "dict.txt"), os.path.join(TEST_PATH, "temp_files", "source.txt"), os.path.join(TEST_PATH, "temp_files", "target.txt"))
        with open(os.path.join(TEST_PATH,"temp_files", "target.txt")) as f:
            self.assertEqual(f.read().strip(), "abril\nabril\nagosto\nagosto")

class PoToCorpusTestCase(unittest.TestCase):

    def setUp(self):
        os.makedirs(os.path.join(TEST_PATH,"temp_files"))

    def tearDown(self):
        logger.info('tear down')
        shutil.rmtree(os.path.join(TEST_PATH,"temp_files"), ignore_errors=True)

    def test_english(self):
        create_corpus_from_po(os.path.join(TEST_PATH, "test_files", "about.po"), os.path.join(TEST_PATH, "temp_files", "source.txt"), os.path.join(TEST_PATH,"temp_files", "target.txt"))
        with open(os.path.join(TEST_PATH,"temp_files","source.txt")) as f:
            self.assertEqual(f.read().strip(), "About MongoDB Documentation\nLicense")

    def test_spanish(self):
        create_corpus_from_po(os.path.join(TEST_PATH, "test_files", "about.po"), os.path.join(TEST_PATH, "temp_files", "source.txt"), os.path.join(TEST_PATH, "temp_files", "target.txt"))
        with open(os.path.join(TEST_PATH,"temp_files", "target.txt")) as f:
            self.assertEqual(f.read().strip(), "Acerca de la documentacion de MongoDB\nLicencia")


class OneCorpusTestCase(unittest.TestCase):

    def __init__(self, *args, **kwargs):
        super(OneCorpusTestCase, self).__init__(*args, **kwargs)
        self.conf_file = os.path.join(TEST_PATH, "configs_test", "one_corpus.yaml")

    def setUp(self):
        self.cconf = ingest_yaml_doc(self.conf_file)
        for source in self.cconf['sources']:
            source['source_file_path'] = os.path.join(TEST_PATH, source['source_file_path'])
            source['target_file_path'] = os.path.join(TEST_PATH, source['target_file_path'])
        self.cconf['container_path'] = os.path.join(TEST_PATH, self.cconf['container_path'])

        self.cconf = CorporaConfig(self.cconf)

    def tearDown(self):
        logger.info('tear down')
        shutil.rmtree(os.path.join(TEST_PATH, "temp_files"), ignore_errors=True)

    def test_english_train(self):
        create_hybrid_corpora(self.cconf)
        with open(os.path.join(TEST_PATH, "temp_files", "train.en-es.en")) as f:
            self.assertEqual(f.read().strip(), "hello")

    def test_spanish_train(self):
        create_hybrid_corpora(self.cconf)
        with open(os.path.join(TEST_PATH,"temp_files", "train.en-es.es")) as f:
            self.assertEqual(f.read().strip(), "hola")

    def test_english_tune(self):
        create_hybrid_corpora(self.cconf)
        with open(os.path.join(TEST_PATH, "temp_files", "tune.en-es.en")) as f:
            self.assertEqual(f.read().strip(), "")

    def test_spanish_tune(self):
        create_hybrid_corpora(self.cconf)
        with open(os.path.join(TEST_PATH, "temp_files", "tune.en-es.es")) as f:
            self.assertEqual(f.read().strip(), "")

    def test_english_test(self):
        create_hybrid_corpora(self.cconf)
        with open(os.path.join(TEST_PATH, "temp_files", "test.en-es.en")) as f:
            self.assertEqual(f.read().strip(), "")

    def test_spanish_test(self):
        create_hybrid_corpora(self.cconf)
        with open(os.path.join(TEST_PATH, "temp_files", "test.en-es.es")) as f:
            self.assertEqual(f.read().strip(), "")


class EvenCorpusTestCase(unittest.TestCase):

    def __init__(self, *args, **kwargs):
        super(EvenCorpusTestCase, self).__init__(*args, **kwargs)
        self.conf_file = os.path.join(TEST_PATH, "configs_test", "even_corpora.yaml")

    def setUp(self):
        self.cconf = ingest_yaml_doc(self.conf_file)
        for source in self.cconf['sources']:
            source['source_file_path'] = os.path.join(TEST_PATH, source['source_file_path'])
            source['target_file_path'] = os.path.join(TEST_PATH, source['target_file_path'])
        self.cconf['container_path'] = os.path.join(TEST_PATH, self.cconf['container_path'])

        self.cconf = CorporaConfig(self.cconf)

    def tearDown(self):
        logger.info('tear down')
        shutil.rmtree(os.path.join(TEST_PATH, "temp_files"), ignore_errors=True)

    def test_english_train(self):
        create_hybrid_corpora(self.cconf)
        with open(os.path.join(TEST_PATH, "temp_files", "train.en-es.en")) as f:
            self.assertEqual(f.read().strip(), "hello\ne1-m\ne2-m\ne3-m")

    def test_spanish_train(self):
        create_hybrid_corpora(self.cconf)
        with open(os.path.join(TEST_PATH, "temp_files", "train.en-es.es")) as f:
            self.assertEqual(f.read().strip(), "hola\ns1-m\ns2-m\ns3-m")

    def test_english_tune(self):
        create_hybrid_corpora(self.cconf)
        with open(os.path.join(TEST_PATH, "temp_files", "tune.en-es.en")) as f:
            self.assertEqual(f.read().strip(), "e4-m\ne5-m\ne6-m")

    def test_spanish_tune(self):
        create_hybrid_corpora(self.cconf)
        with open(os.path.join(TEST_PATH, "temp_files", "tune.en-es.es")) as f:
            self.assertEqual(f.read().strip(), "s4-m\ns5-m\ns6-m")

    def test_english_test(self):
        create_hybrid_corpora(self.cconf)
        with open(os.path.join(TEST_PATH, "temp_files", "test.en-es.en")) as f:
            self.assertEqual(f.read().strip(), "e7-m\ne8-m\ne9-m\ne10-m")

    def test_spanish_test(self):
        create_hybrid_corpora(self.cconf)
        with open(os.path.join(TEST_PATH, "temp_files", "test.en-es.es")) as f:
            self.assertEqual(f.read().strip(), "s7-m\ns8-m\ns9-m\ns10-m")


class HiddenCorpusTestCase(unittest.TestCase):

    def __init__(self, *args, **kwargs):
        super(HiddenCorpusTestCase, self).__init__(*args, **kwargs)
        self.conf_file = os.path.join(TEST_PATH, "configs_test", "hidden_corpora.yaml")

    def setUp(self):
        self.cconf = ingest_yaml_doc(self.conf_file)
        for source in self.cconf['sources']:
            source['source_file_path'] = os.path.join(TEST_PATH, source['source_file_path'])
            source['target_file_path'] = os.path.join(TEST_PATH, source['target_file_path'])
        self.cconf['container_path'] = os.path.join(TEST_PATH, self.cconf['container_path'])

        self.cconf = CorporaConfig(self.cconf)

    def tearDown(self):
        logger.info('tear down')
        shutil.rmtree(os.path.join(TEST_PATH,"temp_files"), ignore_errors=True)

    def test_english_train(self):
        create_hybrid_corpora(self.cconf)
        with open(os.path.join(TEST_PATH, "temp_files", "train.en-es.en")) as f:
            self.assertEqual(f.read().strip(), "hello\ne1-m\ne2-m\ne3-m")

    def test_spanish_train(self):
        create_hybrid_corpora(self.cconf)
        with open(os.path.join(TEST_PATH, "temp_files", "train.en-es.es")) as f:
            self.assertEqual(f.read().strip(), "hola\ns1-m\ns2-m\ns3-m")

    def test_english_tune(self):
        create_hybrid_corpora(self.cconf)
        with open(os.path.join(TEST_PATH, "temp_files", "tune.en-es.en")) as f:
            self.assertEqual(f.read().strip(), "")

    def test_spanish_tune(self):
        create_hybrid_corpora(self.cconf)
        with open(os.path.join(TEST_PATH, "temp_files", "tune.en-es.es")) as f:
            self.assertEqual(f.read().strip(), "")

    def test_english_test(self):
        create_hybrid_corpora(self.cconf)
        with open(os.path.join(TEST_PATH, "temp_files", "test.en-es.en")) as f:
            self.assertEqual(f.read().strip(), "")

    def test_spanish_test(self):
        create_hybrid_corpora(self.cconf)
        with open(os.path.join(TEST_PATH, "temp_files", "test.en-es.es")) as f:
            self.assertEqual(f.read().strip(), "")


class UnevenCorpusTestCase(unittest.TestCase):

    def __init__(self, *args, **kwargs):
        super(UnevenCorpusTestCase, self).__init__(*args, **kwargs)
        self.conf_file = TEST_PATH+"/configs_test/uneven_corpora.yaml"

    def setUp(self):
        self.cconf = ingest_yaml_doc(self.conf_file)
        for source in self.cconf['sources']:
            source['source_file_path'] = os.path.join(TEST_PATH, source['source_file_path'])
            source['target_file_path'] = os.path.join(TEST_PATH, source['target_file_path'])
        self.cconf['container_path'] = os.path.join(TEST_PATH, self.cconf['container_path'])

        self.cconf = CorporaConfig(self.cconf)

    def tearDown(self):
        logger.info('tear down')
        shutil.rmtree(os.path.join(TEST_PATH,"temp_files"), ignore_errors=True)

    def test_english_train(self):
        print self.cconf
        create_hybrid_corpora(self.cconf)
        with open(os.path.join(TEST_PATH, "temp_files", "train.en-es.en")) as f:
            self.assertEqual(f.read().strip(), "e1-l\ne2-l\ne3-l\ne4-l\ne5-l\ne6-l\ne7-l\ne8-l\ne9-l\ne10-l\ne11-l\ne12-l\ne13-l\ne14-l\ne15-l\ne1-m\ne2-m\ne3-m\ne1-m\ne2-m\ne3-m\ne1-m\ne2-m\ne3-m\ne1-m\ne2-m\ne3-m\ne1-m\ne2-m\ne3-m")

    def test_spanish_train(self):
        create_hybrid_corpora(self.cconf)
        with open(os.path.join(TEST_PATH, "temp_files", "train.en-es.es")) as f:
            self.assertEqual(f.read().strip(), "s1-l\ns2-l\ns3-l\ns4-l\ns5-l\ns6-l\ns7-l\ns8-l\ns9-l\ns10-l\ns11-l\ns12-l\ns13-l\ns14-l\ns15-l\ns1-m\ns2-m\ns3-m\ns1-m\ns2-m\ns3-m\ns1-m\ns2-m\ns3-m\ns1-m\ns2-m\ns3-m\ns1-m\ns2-m\ns3-m")

    def test_english_tune(self):
        create_hybrid_corpora(self.cconf)
        with open(os.path.join(TEST_PATH, "temp_files", "tune.en-es.en")) as f:
            self.assertEqual(f.read().strip(), "e16-l\ne17-l\ne18-l\ne19-l\ne4-m\ne5-m\ne6-m\ne7-m\ne4-m\ne5-m")

    def test_spanish_tune(self):
        create_hybrid_corpora(self.cconf)
        with open(os.path.join(TEST_PATH, "temp_files", "tune.en-es.es")) as f:
            self.assertEqual(f.read().strip(), "s16-l\ns17-l\ns18-l\ns19-l\ns4-m\ns5-m\ns6-m\ns7-m\ns4-m\ns5-m")

    def test_english_test(self):
        create_hybrid_corpora(self.cconf)
        with open(os.path.join(TEST_PATH, "temp_files", "test.en-es.en")) as f:
            self.assertEqual(f.read().strip(), "e20-l\ne20-l\ne8-m\ne9-m\ne10-m")

    def test_spanish_test(self):
        create_hybrid_corpora(self.cconf)
        with open(os.path.join(TEST_PATH, "temp_files", "test.en-es.es")) as f:
            self.assertEqual(f.read().strip(), "s20-l\ns20-l\ns8-m\ns9-m\ns10-m")


class BadFileCorpusTestCase(unittest.TestCase):

    def __init__(self, *args, **kwargs):
        super(BadFileCorpusTestCase, self).__init__(*args, **kwargs)
        self.conf_file = os.path.join(TEST_PATH, "configs_test", "bad_file_corpus.yaml")

    def tearDown(self):
        logger.info('tear down')
        shutil.rmtree(os.path.join(TEST_PATH,"temp_files"), ignore_errors=True)

    def test_cconf_creation(self):
        self.cconf = ingest_yaml_doc(self.conf_file)
        for source in self.cconf['sources']:
            source['source_file_path'] = os.path.join(TEST_PATH, source['source_file_path'])
            source['target_file_path'] = os.path.join(TEST_PATH, source['target_file_path'])
        self.cconf['container_path'] = os.path.join(TEST_PATH, self.cconf['container_path'])

        with self.assertRaises(Exception):
            self.cconf = CorporaConfig(self.cconf)

class BadPerc1CorpusTestCase(unittest.TestCase):

    def __init__(self, *args, **kwargs):
        super(BadPerc1CorpusTestCase, self).__init__(*args, **kwargs)
        self.conf_file = os.path.join(TEST_PATH, "configs_test", "bad_perc_corpora.yaml")

    def tearDown(self):
        logger.info('tear down')
        shutil.rmtree(os.path.join(TEST_PATH,"temp_files"), ignore_errors=True)

    def test_cconf_creation(self):
        self.cconf = ingest_yaml_doc(self.conf_file)
        for source in self.cconf['sources']:
            source['source_file_path'] = os.path.join(TEST_PATH, source['source_file_path'])
            source['target_file_path'] = os.path.join(TEST_PATH, source['target_file_path'])
        self.cconf['container_path'] = os.path.join(TEST_PATH, self.cconf['container_path'])

        with self.assertRaises(Exception):
            self.cconf = CorporaConfig(self.cconf)


class BadPerc2CorpusTestCase(unittest.TestCase):

    def __init__(self, *args, **kwargs):
        super(BadPerc2CorpusTestCase, self).__init__(*args, **kwargs)
        self.conf_file = os.path.join(TEST_PATH, "configs_test", "bad_perc_corpora2.yaml")

    def tearDown(self):
        logger.info('tear down')
        shutil.rmtree(os.path.join(TEST_PATH,"temp_files"), ignore_errors=True)

    def test_cconf_creation(self):
        self.cconf = ingest_yaml_doc(self.conf_file)
        for source in self.cconf['sources']:
            source['source_file_path'] = os.path.join(TEST_PATH, source['source_file_path'])
            source['target_file_path'] = os.path.join(TEST_PATH, source['target_file_path'])
        self.cconf['container_path'] = os.path.join(TEST_PATH, self.cconf['container_path'])

        with self.assertRaises(Exception):
            self.cconf = CorporaConfig(self.cconf)


class BadPerc3CorpusTestCase(unittest.TestCase):

    def __init__(self, *args, **kwargs):
        super(BadPerc3CorpusTestCase, self).__init__(*args, **kwargs)
        self.conf_file = os.path.join(TEST_PATH, "configs_test", "bad_perc_corpora3.yaml")

    def tearDown(self):
        logger.info('tear down')
        shutil.rmtree(os.path.join(TEST_PATH,"temp_files"), ignore_errors=True)

    def test_cconf_creation(self):
        self.cconf = ingest_yaml_doc(self.conf_file)
        for source in self.cconf['sources']:
            source['source_file_path'] = os.path.join(TEST_PATH, source['source_file_path'])
            source['target_file_path'] = os.path.join(TEST_PATH, source['target_file_path'])
        self.cconf['container_path'] = os.path.join(TEST_PATH, self.cconf['container_path'])

        with self.assertRaises(Exception):
            self.cconf = CorporaConfig(self.cconf)
if __name__ == '__main__':
    unittest.main()



# Copyright 2014 MongoDB, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os.path
import logging
import datetime

logger = logging.getLogger('giza.config.jeerah')

from giza.config.base import ConfigurationBase, RecursiveConfigurationBase
from giza.config.runtime import RuntimeStateConfigurationBase

def fetch_config(args):
    c = JeerahConfig()
    c.ingest(args.conf_path)
    c.runstate = args

    return c

class JeerahRuntimeStateConfig(RuntimeStateConfigurationBase):
    _option_registry = ['project', 'user_conf_path']

    @property
    def sprint(self):
        if 'sprint' in self.state:
            return self.state['sprint']
        else:
            return 'current'

    @sprint.setter
    def sprint(self, value):
        self.state['sprint'] = value

    @property
    def conf_path(self):
        if 'conf_path' not in self.state:
            self.conf_path = None

        return self.state['conf_path']

    @conf_path.setter
    def conf_path(self, value):
        if value is not None and os.path.exists(value):
            self.state['conf_path'] = value
        else:
            try:
                self._discover_conf_file('.scrumpy.yaml')
            except OSError:
                self._discover_conf_file('scrumpy.yaml')
            except OSError:
                logger.error('could not find scrumpy config file.')
                raise OSError


class JeerahConfig(ConfigurationBase):
    @property
    def runstate(self):
        return self.state['runstate']

    @runstate.setter
    def runstate(self, value):
        if isinstance(value, JeerahRuntimeStateConfig):
            value.conf = self
            self.state['runstate'] = value
        else:
            msg = "invalid runtime state"
            logger.critical(msg)
            raise TypeError(msg)

    @property
    def buckets(self):
        if 'buckets' in self.state:
            return self.state['buckets']
        else:
            return {}

    @buckets.setter
    def buckets(self, value):
        if isinstance(value, dict):
            if 'buckets' in self.state:
                self.state['buckets'].update(value)
            else:
                self.state['buckets'] = value
        else:
            raise TypeError('{0} is not a dict'.format(value))

    @property
    def sprints(self):
        return self.state['sprints']

    @sprints.setter
    def sprints(self, value):
        if isinstance(value, SprintCollectionConfig):
            self.state['sprints'] = value
        else:
            self.state['sprints'] = SprintCollectionConfig(value)

    @property
    def site(self):
        return self.state['site']

    @site.setter
    def site(self, value):
        if isinstance(value, JeerahSiteConfig):
            self.state['site'] = value
        else:
            self.state['site'] = JeerahSiteConfig(value)

    @property
    def reporting(self):
        return self.state['reporting']

    @reporting.setter
    def reporting(self, value):
        if isinstance(value, ReportingConfig):
            self.state['reporting'] = value
        else:
            self.state['reporting'] = ReportingConfig(value)

    @property
    def modification(self):
        return self.state['modification']

    @modification.setter
    def modification(self, value):
        if isinstance(value, ModificationConfig):
            self.state['modification'] = value
        else:
            self.state['modification'] = ModificationConfig(value, self)

class SprintCollectionConfig(ConfigurationBase):
    def ingest(self, obj):
        if isinstance(obj, dict):
            for k, v in obj.items():
                self._add_sprint(k, v)
        elif isinstance(obj, list):
            for item in obj:
                self._add_sprint(item['name'], item)
        else:
            logger.error('{0} is malformed sprint description'.format(obj))

    def _add_sprint(self, name, sprint):
        # compatibility shim:
        if isinstance(sprint, list):
            rendered_sprint = {
                'name': name,
                'fix_versions': sprint,
            }
        else:
            rendered_sprint = sprint

        # commit object to interface's storage
        try:
            self.state[name] = SprintConfig(rendered_sprint)
        except:
            logger.error('{0} is not a valid sprint'.format(name))

    def get_sprint_versions(self, name):
        if name in self.state:
            return self.state[name].fix_versions
        else:
            logger.error("sprint '{0}' does not exist".format(name))

    def get_sprint(self, name):
        if name in self.state:
            return self.state[name]
        else:
            logger.error("sprint '{0}' does not exist".format(name))

class SprintConfig(ConfigurationBase):
    _option_registry = [ 'fix_versions', 'name', 'staffing' ]

    @property
    def quota(self):
        if 'quota' in self.state:
            return self.state['quota']
        elif 'staffing' in self:
            self.quota = sum([ v for v in self.staffing.values() ])
            return self.state['quota']
        else:
            return None

    @quota.setter
    def quota(self, value):
        if isinstance(value, (int, float, complex)):
            self.state['quota'] = value
        else:
            raise TypeError('{0} is not a valid sprint quota')

    @property
    def start(self):
        if 'start' in self.state:
            return self.state['start']
        else:
            return None

    @start.setter
    def start(self, value):
        if isinstance(value, datetime.date):
            self.state['start'] = value
        else:
            try:
                self.state['start'] = datetime.datetime.strptime(value, "%Y-%m-%d")
            except:
                logger.warning('{0} is not a properly formed date. (use YYYY-MM-DD)'.format(value))

    @property
    def end(self):
        if 'end' in self.state:
            return self.state['end']
        else:
            return None

    @end.setter
    def end(self, value):
        if isinstance(value, datetime.date):
            self.state['end'] = value
        else:
            try:
                self.state['end'] = datetime.datetime.strptime(value, "%Y-%m-%d")
            except:
                logger.warning('{0} is not a properly formed date. (use YYYY-MM-DD)'.format(value))


class JeerahSiteConfig(ConfigurationBase):
    _option_registry = ['url']

    @property
    def credentials(self):
        return self.state['credentials']

    @credentials.setter
    def credentials(self, value):
        value = os.path.expanduser(value)
        self.state['credentials'] = value

    @property
    def projects(self):
        return self.state['project']

    @projects.setter
    def projects(self, value):
        if isinstance(value, list):
            self.state['project'] = value
        else:
            self.state['project'] = [value]

class ReportingConfig(ConfigurationBase):
    @property
    def units(self):
        if 'units' not in self.state:
            return 'hours'
        else:
            return self.state['units']

    @units.setter
    def units(self, value):
        possible_values = ('days', 'hours', 'count')
        if value in possible_values:
            self.state['units'] = value
        else:
            raise TypeError('{0} is not in {1}'.format(value, possible_values))

    @property
    def format(self):
        if 'format' not in self.state:
            return 'json'
        else:
            return self.state['format']

    @format.setter
    def format(self, value):
        if value in ('json', 'yaml'):
            self.state['format'] = value

class ModificationConfig(RecursiveConfigurationBase):
    @property
    def mirroring(self):
        return self.state['mirroring']

    @mirroring.setter
    def mirroring(self, value):
        if isinstance(value, ProjectMirroringConfig):
            self.state['mirroring'] = value
        else:
            self.state['mirroring'] = ProjectMirroringConfig(value, self.conf)

class ProjectMirroringConfig(RecursiveConfigurationBase):
    @property
    def source(self):
        # can't validate this during injestion/setting because self.conf may not
        # point to a populated config object yet.

        value = self.state['source']

        if value not in self.conf.site.projects:
            logger.warning('{0} is not a valid project'.format(value))

        return value

    @source.setter
    def source(self, value):
        self.state['source'] = value

    @property
    def target(self):
        # can't validate this during injestion/setting because self.conf may not
        # point to a populated config object yet.

        value = self.state['target']

        for project in value:
            if project not in self.conf.site.projects:
                logger.warning('{0} is not in the list of projects ({1})'.format(project, self.conf.site.projects))

        return value

    @target.setter
    def target(self, value):
        if isinstance(value, list):
            self.state['target'] = value
        else:
            self.state['target'] = [value]



# Copyright 2014 MongoDB, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
Steps schema is: ::

   {
     "title": <str>,
     "stepnum": <int>,
     "pre": <str>,
     "post": <str>,
     "ref": <str>,
     "action": {
                  "heading": <str>,
                  "code": <str>,
                  "language": <str>,
                  "content": <str>,
                  "pre": <str>,
                  "post": <str>
                }
   }

Notes:

- stepnum is optional. If not specified, we assume that the sequence starts at
  one. If you specify your own ``stepnum`` in one step you have to specify your
  own step number in all steps in this sequence. The script enforces order, so
  that if you specify stepnum, you don't need to specify steps in the source file in order.

- "title" and "heading" fields can optionally hold a document that contains
  both a "text" field *and* a "character" field if you need to adjust the level
  of the heading. For example: ::

    {
      "title":
        {
          "text": "name of step",
          "character": "-"
        }
    }

  Therefore "name of step" is, by MongoDB docs convention an "h2". By default,
  heading within actions are h4s and titles of steps are h3s.

- pre/post are optional. and allow you to add prefix or postfix text to a step
  or code example/action.

- Action should be either a doc or a list of docs. Their fields are optional,
  with the following notable points:

  - "language" refers to the syntax highlighting of "code," and is unused
    otherwise.

  - "content" is a paragraph, for steps that don't have code examples.

- the spec/agg format would be at least: ::

    {
       "source":
         {
           "file": <str>
           "ref":
         }
    }

  callers may specify ``description`` and ``title`` to override the source
  location. (ref needs to be modified in calling location.)

There are several situations where we raise errors because a step document is
invalid:

1. A step has both a "source" (i.e. is an included step), and an "action" field.

2. A code block contains both "content" and "code" fields.

"""



# Copyright 2014 MongoDB, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
Post-processes Sphinx's latex output and generate PDFs from these ``tex`` files.
"""

import logging
import os
import re
import subprocess

logger = logging.getLogger('giza.content.post.latex')

from giza.content.helper import edition_check
from giza.tools.command import command
from giza.tools.serialization import ingest_yaml_list
from giza.tools.strings import hyph_concat
from giza.tools.transformation import process_page
from giza.tools.files import (create_link, copy_if_needed,
                              decode_lines_from_file, encode_lines_to_file)

#################### PDFs from Latex Produced by Sphinx  ####################

def _render_tex_into_pdf(fn, deployed_path, path, output_format="pdf"):
    """
    Runs ``pdflatex`` operations, can generate ``dvi`` and ``pdf``. Runs
    pdflatex multiple times to correctly index and cross reference the PDF.
    """

    if output_format == 'dvi':
        pdflatex = 'TEXINPUTS=".:{0}:" pdflatex --output-format dvi --interaction batchmode --output-directory {0} {1}'.format(path, fn)
    elif output_format == 'pdf':
        pdflatex = 'TEXINPUTS=".:{0}:" pdflatex --interaction batchmode --output-directory {0} {1}'.format(path, fn)
    else:
        logger.error('not rendering pdf because {0} is not an output format'.format(output_format))
        return

    base_fn = os.path.basename(fn)
    cmds = [ pdflatex,
             "makeindex -s {0}/python.ist {0}/{1}.idx ".format(path, base_fn[:-4]),
             pdflatex,
             pdflatex ]

    if output_format == 'dvi':
        cmds.append("cd {0}; dvipdf {1}.dvi".format(path, base_fn[:-4]))

    for idx, cmd in enumerate(cmds):
        r = command(command=cmd, ignore=True)

        if r.succeeded is True:
            logger.info('pdf completed rendering stage {0} of {1} successfully ({2}).'.format(idx, len(cmds), base_fn))
        else:
            if idx <= 1:
                logger.warning('pdf build encountered error early on {0}, continuing cautiously.'.format(base_fn))
                continue
            else:
                logger.error('pdf build encountered error running pdflatex, investigate on {0}. terminating'.format(base_fn))
                logger.error(cmd)
                return False

    pdf_fn = os.path.splitext(fn)[0] + '.pdf'
    copy_if_needed(pdf_fn, deployed_path, 'pdf')

def pdf_tasks(sconf, conf, app):
    """Adds tasks to a BuildApp() to generate all PDFs."""

    target = sconf.builder
    if 'pdfs' not in conf.system.files.data:
        return

    # a list of tuples in (compileRegex, substitution) format.
    tex_regexes = [ ( re.compile(r'(index|bfcode)\{(.*)--(.*)\}'),
                      r'\1\{\2-\{-\}\3\}'),
                    ( re.compile(r'\\PYGZsq{}'), "'"),
                    ( re.compile(r'\\code\{/(?!.*{}/|etc|usr|data|var|srv|data|bin|dev|opt|proc|24|private)'),
                      r'\code{' + conf.project.url + r'/' + conf.project.tag + r'/') ]

    # the ordering of tasks. First post-process the tex generated by sphinx:
    process_app = app.add('app')
    # then convert tex to pdf
    render_app = app.add('app')
    # then migrate to build/public/
    migrate_app = app.add('app')
    # then create symlinks for alternate named files.
    link_app = app.add('app')

    # the path that sphinx writes tex files to are are different for editions.
    if 'edition' in conf.project and conf.project.edition != conf.project.name:
        latex_dir = os.path.join(conf.paths.projectroot, conf.paths.branch_output, hyph_concat(target, conf.project.edition))
    else:
        latex_dir = os.path.join(conf.paths.projectroot, conf.paths.branch_output, target)

    deploy_path = os.path.join(conf.paths.projectroot, conf.paths.public_site_output)

    # special case operations on "offset pdfs", which use EPS images.
    if 'tags' in sconf and "offset" in sconf.tags:
        output_format = "dvi"
        sty_file = os.path.join(latex_dir, 'sphinx.sty')
        process_page(fn=sty_file,
                     output_fn=sty_file,
                     regex=(re.compile(r'\\usepackage\[pdftex\]\{graphicx\}'), r'\usepackage{graphicx}'),
                     app=process_app,
                     builder='sphinx-latex',
                     copy='ifNeeded')
    else:
        output_format = "pdf"

    for i in conf.system.files.data.pdfs:
        if edition_check(i, conf) is False:
            continue

        #compatibility shim for new/old images
        i = i.dict()
        tagged_name = i['output'][:-4] + '-' + i['tag']
        deploy_fn = tagged_name + '-' + conf.git.branches.current + '.pdf'
        link_name = deploy_fn.replace('-' + conf.git.branches.current, '')

        i['source'] = os.path.join(latex_dir, i['output'])
        i['processed'] = os.path.join(latex_dir, tagged_name + '.tex')
        i['pdf'] = os.path.join(latex_dir, tagged_name + '.pdf')
        i['deployed'] = os.path.join(deploy_path, deploy_fn)
        i['link'] = os.path.join(deploy_path, link_name)
        i['path'] = latex_dir

        # add the processing task
        process_page(i['source'], i['processed'], tex_regexes, process_app, builder='tex-munge', copy='ifNeeded')

        # add task for changing TEX to PDF.
        render_task = render_app.add('task')
        render_task.dependency = None #i['processed']
        render_task.target = i['pdf']
        render_task.job = _render_tex_into_pdf
        render_task.args = (i['processed'], i['deployed'], i['path'], output_format)

        # if needed create links.
        if i['link'] != i['deployed']:
            link_task = link_app.add('task')
            link_task.dependency = i['deployed']
            link_task.target = i['link']
            link_task.job = create_link
            link_task.args = (deploy_fn, i['link'])



# Copyright 2014 MongoDB, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
Controls the generation of the ``meta/includes.txt`` file which provides a
full-overview of all included files, along with an easy generation of
:mod:`giza.includes` data to make it easy to undersatnd the relationship between
include files and their use. For use in documentation development only.
"""

import logging
import os.path

logger = logging.getLogger('giza.content.includes')

from giza.includes import generated_includes, included_recusively, include_files
from giza.tools.files import expand_tree
from giza.tools.serialization import ingest_yaml_doc
from giza.tools.timing import Timer

from rstcloth.rstcloth import RstCloth

suppressed_page_prefixes = [
    '/includes/generated',
    '/includes/table',
    '/includes/steps',
    '/includes/install',
    '/includes/manpage',
    '/includes/toc',
    '/includes/ref-toc',
    '/includes/ref-spec',
    '/includes/metadata',
]

def write_include_index(overview_fn, conf):
    fd = include_file_data(conf)
    r = build_page(fd, conf)

    if r is not None:
        r.write(overview_fn)
        logger.info('includes: generated /meta/includes source page.')

def include_file_data(conf):
    inc_path = os.path.join(conf.paths.includes)
    include_file_list = expand_tree(path=inc_path, input_extension=None)
    include_graph = include_files(conf=conf)

    recursive_use = included_recusively(conf, include_graph)
    generated = generated_includes(conf)

    omni = {}
    for idx, fn in enumerate(include_file_list):
        incf = fn[len(conf.paths.source):]

        if fn.endswith('~'):
            continue

        for prefix in suppressed_page_prefixes:
            if incf.startswith(prefix):
                break
        else:
            omni[incf] = {
                'id': idx,
                'name': os.path.splitext(incf)[0],
                'path': incf,
            }

            if incf in generated:
                omni[incf]['generated'] = True
            else:
                omni[incf]['generated'] = False

            if incf in recursive_use:
                omni[incf]['recursive'] = True
            else:
                omni[incf]['recursive'] = False

            if incf in include_graph:
                omni[incf]['num_clients'] = len(include_graph[incf])

                omni[incf]['clients'] = []
                for cl in include_graph[incf]:
                    cl, ext = os.path.splitext(cl)

                    if ext == 'yaml':
                        continue
                    if (cl.startswith('/includes/generated/overview') or
                        cl.startswith('/includes/manpage-')):
                        continue

                    omni[incf]['clients'].append(cl)

                if len(omni[incf]['clients']) == 0:
                    omni[incf]['yaml_only'] = True
                else:
                    omni[incf]['yaml_only'] = False
            else:
                omni[incf]['clients'] = dict()
                omni[incf]['num_clients'] = 0

            with open(fn, 'r') as f:
                omni[incf]['content'] = [ ln.rstrip() for ln in f.readlines() ]

    return omni

def build_page(data, conf):
    if 'includes' not in conf.system.files.data:
        return
    else:
        iconf = conf.system.files.data.includes

    r = RstCloth()

    r.title(iconf['title'])
    r.newline()
    r.directive('default-domain', iconf['domain'])
    r.newline()

    try:
        r.content(iconf['introduction'])
        r.newline()
    except KeyError:
        logger.debug('include meta file lacks an introduction.')

    r.directive(name='contents', arg='Included Files',
                fields=[ ('backlinks', 'none'),
                         ('class', 'long-toc'),
                         ('depth', 1),
                         ('local', ''),
                       ])
    r.newline()

    data = data.items()
    data.sort()
    for _, record in data:
        page_name = r.pre(record['name'])
        r.heading(text=page_name, char='-', indent=0)
        r.newline()

        r.heading('Meta', char='~', indent=0)
        r.newline()

        if record['num_clients'] == 0:
            r.content('{0} is not included in any files.'.format(page_name))

            r.newline()
            add_content(r, record)

        elif record['num_clients'] == 1:
            if record['yaml_only']:
                r.content('{0} is only included in yaml files.'.format(page_name))
                r.newline()
            else:
                link = r.role('doc', record['clients'][0])
                r.content('{0} is only included in {1}.'.format(page_name,  link))
                r.newline()

            add_meta(r, page_name, record)

            add_content(r, record)
        else:
            r.content('{0} is included in **{1}** files.'.format(page_name, record['num_clients']),
                      wrap=False)
            r.newline()

            add_meta(r, page_name, record)

            if record['yaml_only'] is False:
                clients = [ p for p in
                            record['clients']
                            if not p.startswith('/includes')
                            ]

                if len(clients) == 1:
                    client_link = r.role('doc', clients[0])

                    inc_str = '{0} is the only file that includes {1} that is not also an include.'
                    r.content(inc_str.format(client_link, page_name))

                    r.newline()
                else:
                    r.heading('Client Pages', char='~', indent=0)
                    r.newline()

                    for pg in clients:
                        client_link = r.role('doc', pg)

                        r.li(client_link, wrap=False)
                        r.newline()

            add_include_example(r, page_name, record['path'])
            add_content(r, record)

    return r

def add_include_example(r, name, path):
    r.heading('Example Use', char='~', indent=0)

    r.content('To include {0} in a document, use the following statement:'.format(name))
    r.newline()

    r.codeblock(content='.. include:: {0}'.format(path),
                language='rst')
    r.newline()

def add_content(r, record):
    r.heading('Content', char='~', indent=0)
    r.newline()
    r.codeblock(content=record['content'],
                language=os.path.splitext(record['path'])[1][1:],
                wrap=False)
    r.newline()

meta_strs = {
    'recursive': '{0} is included in another file used as an include.',
    'generated': '{0} is a generated file.'
}

def add_meta(r, page_name, record):
    for i in meta_strs.keys():
        if record[i] is True:
            r.content(meta_strs[i].format(page_name))
            r.newline()

def includes_tasks(conf, app):
    includes_dir = os.path.join(conf.paths.projectroot, conf.paths.branch_includes)
    meta_dir = os.path.join(conf.paths.projectroot, conf.paths.branch_source, 'meta')

    if os.path.exists(includes_dir) and os.path.exists(meta_dir):
        overview_fn = os.path.join(conf.paths.projectroot,
                                   conf.paths.branch_includes,
                                   'generated',
                                   'overview.rst')

        t = app.add('task')
        t.job = write_include_index
        t.target = overview_fn
        t.dependency = [os.path.join(includes_dir, fn) for fn in os.listdir(includes_dir)]
        t.args = [overview_fn, conf]
